#! /bin/bash
# ----------------------------------------------------------------------------
# checkForChanges.sh - Check which package should be build and trigger
#                      the corresponding build job
#
# Creation   :  2013-04-19  starwarsfan
#
# Copyright (c) 2013-2018 the eisfair team, team(at)eisfair(dot)org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> /tmp/checkForChanges$$.log
#set -x

callDir=$(pwd)
ownLocation="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
scriptName=$(basename $0)
cd ${ownLocation}
. ./include/helpers_log.sh
rtc=0

javaMinHeap='256M'
javaMaxHeap='512M'

# -----------------------------
# Check if settings file exists
if [ -f "settings.txt" ] ; then
    # ----------------------------------------------------------------
    # This should be the normal case: settings.txt exists, so load it.
    . settings.txt
    info "Settings loaded."
elif [ -f "settings.default.txt" ] ; then
    warning ""
    warning "Settings file not existing, creating new one out of default file."
    warning "The new one is on .gitignore, so it is secured that personal settings"
    warning "are not commited to the repository."
    cp settings.default.txt settings.txt
    warning "The script will be aborted here, please modify"
    warning "    ${ownLocation}/settings.txt"
    warning "to fit your needs. If the settings are OK, restart the script."
    warning ""
    die 2 ""
else
    # ---------------------------------------------------------------------
    # Thats odd: Not even 'settings.txt' nor 'settings.default.txt' exists!
    error ""
    error "ERROR: No settings file existing!"
    error "The default file 'settings.default.txt' must exist on the folder"
    error "'_ADMIN/'. This file is used to create"
    error "a personal settings file, which you can setup to your needs."
    error "Please check for that and restart the script again."
    error ""
    die 1 ""
fi



for currentFile in $(git diff --name-only @{1}..) ; do
    # Check for changes but skip files on repo root and skip folder _ADMIN
    if [ "${currentFile%%/*}" != "${currentFile}" -a "${currentFile%%/*}" != '_ADMIN' ] ; then
        echo ${currentFile%%/*} >> /tmp/determinedFolders-$$.txt
    fi
done

echo "=============================================================================="
if [ -e /tmp/determinedFolders-$$.txt ] ; then
    if [ ! -f "$jenkinsPasswordFile" ] ; then
        die 10 "File $jenkinsPasswordFile with the password for user $jenkinsUser does not exist!"
    elif [ ! -f "$jenkinsCliJar" ] ; then
        die 11 "File $jenkinsCliJar not found!"
    else
        # Wipe out duplicate folder entries
        sort -u /tmp/determinedFolders-$$.txt > /tmp/determinedFoldersUnique-$$.txt

        while read packageToTrigger ; do
            info "Triggering build of package '$packageToTrigger':"
            buildJobNamePrefix=${jobNamePrefix}__${packageToTrigger}__
            jobTemplates=$(java -Xms${javaMinHeap} \
                                -Xmx${javaMaxHeap} \
                                -jar ${jenkinsCliJar} \
                                -s ${jenkinsUrl} \
                                list-jobs \
                                --username ${jenkinsUser} \
                                --password-file ${jenkinsPasswordFile} | grep ${buildJobNamePrefix} | tr '\n' ' ')
            for currentBuildJob in ${jobTemplates} ; do
                info -n " - Job $currentBuildJob "
                java -Xms${javaMinHeap} \
                     -Xmx${javaMaxHeap} \
                     -jar ${jenkinsCliJar} \
                     -s ${jenkinsUrl} \
                     build ${currentBuildJob} \
                     --username ${jenkinsUser} \
                     --password-file ${jenkinsPasswordFile}
            done
            info "Finished triggering '$packageToTrigger'"
        done < /tmp/determinedFoldersUnique-$$.txt

        rm -f /tmp/determinedFoldersUnique-$$.txt /tmp/determinedFolders-$$.txt
    fi
else
    info "No changes on at least one of the package directories"
fi
info "=============================================================================="
