#!/bin/bash
# ----------------------------------------------------------------------------
# /var/install/config.d/inadyn-update.sh - paramater update script
#
# Creation   : 2011-02-12 starwarsfan
# Copyright (c) 2011-2017 the eisfair team, <team(at)eisfair(dot)org>
#
# ----------------------------------------------------------------------------

#exec 2> `pwd`/inadyn-update-trace$$.log
#set -x

# Include configlib for using printvar
. /var/install/include/configlib

# Include eislib
. /var/install/include/eislib

# Set package name
packageName=cui-inadyn

# ----------------------------------------------------------------------------
# Set the default values for configuration
START_INADYN='no'

INADYN_UPDATE_INTERVAL='600'
INADYN_ACCOUNT_N=1
INADYN_ACCOUNT_1_NAME='Account 1'
INADYN_ACCOUNT_1_ACTIVE='yes'
INADYN_ACCOUNT_1_SYSTEM='dynamic'
INADYN_ACCOUNT_1_USE_SSL='yes'
INADYN_ACCOUNT_1_IP_SERVER='checkip.two-dns.de'
INADYN_ACCOUNT_1_IP_IP_CHECK_CMD=''
INADYN_ACCOUNT_1_USER_N=1
INADYN_ACCOUNT_1_USER_1_NAME=''
INADYN_ACCOUNT_1_USER_1_PASSWORD=''
INADYN_ACCOUNT_1_USER_1_ALIAS_N=1
INADYN_ACCOUNT_1_USER_1_ALIAS_1='test.homeip.net'
INADYN_MAIL_ON_UPDATE='no'
INADYN_MAIL_TO=''
INADYN_LOGFILE=''
INADYN_LOG_LEVEL='info'


# ----------------------------------------------------------------------------
# Read old configuration and rename old variables
renameOldVariables()
{
    # read old values
    [ -f /etc/config.d/${packageName} ] && . /etc/config.d/${packageName}
}

# ----------------------------------------------------------------------------
# Write config and default files
makeConfigFile()
{
    internal_conf_file=${1}
    {
    # ------------------------------------------------------------------------
    printgpl --conf ${packageName}
    # ------------------------------------------------------------------------

    # ------------------------------------------------------------------------
    printgroup "Basic configuration"
    # ------------------------------------------------------------------------
    printvar 'START_INADYN' 'Use: yes or no'

    printvar 'INADYN_UPDATE_INTERVAL'                    "Interval to check for changed IP"
    printvar 'INADYN_ACCOUNT_N' ''

    idx=1
    while [ "${idx}" -le "${INADYN_ACCOUNT_N}" ] ; do
        printvar "INADYN_ACCOUNT_${idx}_NAME"            "Just a name for the account, used on status mail"
        printvar "INADYN_ACCOUNT_${idx}_ACTIVE"          "Is this account active or not"
        printvar "INADYN_ACCOUNT_${idx}_SYSTEM"          "dynamic, static, custom, zoneedit or no-ip"
        printvar "INADYN_ACCOUNT_${idx}_USE_SSL"         "Use encryption if provider supports it"
        printvar "INADYN_ACCOUNT_${idx}_IP_SERVER"       "checkip.two-dns.de, checkip.dyndns.com or something like that"
        printvar "INADYN_ACCOUNT_${idx}_IP_CHECK_CMD"    "Command to execute to check own IP"

        printvar "INADYN_ACCOUNT_${idx}_USER_N"          "Amount of user accounts"
        eval userAmount='${INADYN_ACCOUNT_'${idx}'_USER_N}'
        userAmount=${userAmount:-0} # Set to 0 if empty
        idx2=1
        while [ "${idx2}" -le "${userAmount}" ] ; do
            printvar "INADYN_ACCOUNT_${idx}_USER_${idx2}_NAME"            ""
            printvar "INADYN_ACCOUNT_${idx}_USER_${idx2}_PASSWORD"        ""
            printvar "INADYN_ACCOUNT_${idx}_USER_${idx2}_ALIAS_N"         ""

            eval aliasAmount='${INADYN_ACCOUNT_'${idx}'_USER_'${idx2}'_ALIAS_N}'
            aliasAmount=${aliasAmount:-0} # Set to 0 if empty

            idx3=1
            while [ "${idx3}" -le "${aliasAmount}" ] ; do
                printvar "INADYN_ACCOUNT_${idx}_USER_${idx2}_ALIAS_${idx3}" "test.homeip.net"
                idx3=$((idx3+1))
            done

            idx2=$((idx2+1))
        done
        idx=$((idx+1))
    done

    printvar "INADYN_MAIL_ON_UPDATE"  "Send mail if IP was updated"
    printvar "INADYN_MAIL_TO"         "Mail recipient"
#    printvar "INADYN_LOGFILE"         "Path and name of logfile. If empty, log to syslog"
    printvar "INADYN_LOG_LEVEL"       "0=standard ... 5=debug"

    # ------------------------------------------------------------------------
    printend
    # ------------------------------------------------------------------------

    } > ${internal_conf_file}
    # Set rights
    chmod 0600 ${internal_conf_file}
    chown root ${internal_conf_file}
}

# ----------------------------------------------------------------------------
# Create the check.d file
makeCheckFile()
{
    printgpl --check ${packageName} >/etc/check.d/${packageName}
    cat >> /etc/check.d/${packageName} <<EOFG
# Variable                                OPT_VARIABLE                 VARIABLE_N                        VALUE
START_INADYN                              -                            -                                 YESNO

INADYN_UPDATE_INTERVAL                    START_INADYN                 -                                 NUMERIC
INADYN_ACCOUNT_N                          START_INADYN                 -                                 NUMERIC
INADYN_ACCOUNT_%_NAME                     START_INADYN                 INADYN_ACCOUNT_N                  NOTEMPTY
INADYN_ACCOUNT_%_ACTIVE                   START_INADYN                 INADYN_ACCOUNT_N                  YESNO
INADYN_ACCOUNT_%_SYSTEM                   INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_N                  INADYN_SYSTEM_CUI
INADYN_ACCOUNT_%_USE_SSL                  INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_N                  YESNO
INADYN_ACCOUNT_%_IP_SERVER                INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_N                  INADYN_IP_SERVER
INADYN_ACCOUNT_%_IP_CHECK_CMD             INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_N                  NONE
INADYN_ACCOUNT_%_USER_N                   INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_N                  NOTEMPTY
INADYN_ACCOUNT_%_USER_%_NAME              INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_%_USER_N           NOTEMPTY
INADYN_ACCOUNT_%_USER_%_PASSWORD          INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_%_USER_N           PASSWD
INADYN_ACCOUNT_%_USER_%_ALIAS_N           INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_%_USER_N           NUMERIC
INADYN_ACCOUNT_%_USER_%_ALIAS_%           INADYN_ACCOUNT_%_ACTIVE      INADYN_ACCOUNT_%_USER_%_ALIAS_N   INADYN_ALIAS
INADYN_MAIL_ON_UPDATE                     START_INADYN                 -                                 YESNO
INADYN_MAIL_TO                            INADYN_MAIL_ON_UPDATE        -                                 MAILADDR
#INADYN_LOGFILE                            START_INADYN                 -                                 E_ABS_PATH
INADYN_LOG_LEVEL                          START_INADYN                 -                                 INADYN_DEBUGLEVEL_CUI

EOFG

    # Set rights for check.d file
    chmod 0600 /etc/check.d/${packageName}
    chown root /etc/check.d/${packageName}

    printgpl --check_exp ${packageName} >/etc/check.d/${packageName}.exp
    knownSystems="dynamic|static|custom|http://www.dyndns.org|http://www.dyn.com|https://freedns.afraid.org|https://www.zoneedit.com|http://www.no-ip.com|http://www.easydns.com|http://www.tzo.com|http://www.3322.org|https://www.dnsomatic.com|http://dns.he.net|http://www.tunnelbroker.net|http://www.dynsip.org|http://www.sitelutions.com|http://www.dnsexit.com|http://www.changeip.com|http://www.zerigo.com|http://www.dhis.org|https://nsupdate.info|https://duckdns.org|https://www.loopia.com|https://www.namecheap.com|https://domains.google.com|https://www.ovh.com|https://www.dtdns.com|http://giradns.com|https://www.duiadns.net|https://ddnss.de|http://dynv6.com|http://ipv4.dynv6.com|https://spdyn.de|https://www.strato.com|https://freemyip.com"
    knownSystemsList="${knownSystems//|/, }"
    cat >> /etc/check.d/${packageName}.exp <<EOFG

INADYN_SYSTEM_CUI     = '${knownSystems}'
                      : 'One of the values ${knownSystemsList} must be used'

INADYN_ALIAS          = '(RE:FQDN)|[1-5]'
                      : 'Only a full qualified domain name or numbers from 1-5 are supportet!'

INADYN_IP_SERVER      = '(RE:FQDN)|(RE:IPADDR)'
                      : 'Only a full qualified domain name or an IP is supportet!'

INADYN_DEBUGLEVEL_CUI = 'none|err|info|notice|debug'
                      : 'Debug level must be one of none, err, info, notice, debug'

EOFG

    # Set rights for check.exp file
    chmod 0600 /etc/check.d/${packageName}.exp
    chown root /etc/check.d/${packageName}.exp

    printgpl --check_ext ${packageName} >/etc/check.d/${packageName}.ext
    cat >> /etc/check.d/${packageName}.ext <<EOFG

set mail_used = "no"

if (inadyn_mail_on_update == "yes")
then
    set mail_used = "yes"
fi


if ( mail_used == "yes" )
then
    stat ("/usr/sbin/sendmail", test)
    if ("\$test_res" != "OK")
    then
        error "A mail package is required to enable sending status mails on update!"
    fi
fi
EOFG

    # Set rights for check.ext file
    chmod 0600 /etc/check.d/${packageName}.ext
    chown root /etc/check.d/${packageName}.ext
}



# ----------------------------------------------------------------------------
# Main
makeConfigFile /etc/default.d/${packageName}

# Update from old version
renameOldVariables

# Write new config file
makeConfigFile /etc/config.d/${packageName}

# Write check.d file
makeCheckFile

exit 0
