#!/bin/bash
# ----------------------------------------------------------------------------
# /var/install/config.d/cui-inadyn.sh - INADYN configuration
#
# Copyright (c) 2008-2010 Rene Hanke, hanker(at)rpg-domain(dot)de
# Copyright (c) 2011-2017 the eisfair team, <team(at)eisfair(dot)org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the F$inadyn_activeoundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> `pwd`/inadyn-trace$$.log
#set -x

# ----------------------------------------------------------------------------
# Include eislib
. /var/install/include/eislib

# Include config
. /etc/config.d/cui-inadyn

writeHeader(){
    local inadyn_configfile=$1
    local inadyn_update_interval=$2
    cat > ${inadyn_configfile} <<EOF
# In-A-Dyn v2.1 configuration file format
period          = ${inadyn_update_interval}

EOF
}

writeProviderEntry(){
    local inadyn_configfile=$1
    local providerPrefix=$2
    local provider=$3
    local username=$4
    local password=$5
    local hostname=$6
    local ssl=$7
    local checkIpSSL=$8
    local checkIpServer=$9
    local checkIpCommand=${10}

    if [ -n "${username}" ] ; then
        username="username        = ${username}"
    else
        username="#username        = ${username}"
    fi
    if [ -n "${password}" ] ; then
        password="password        = ${password}"
    else
        password="#password        = ${password}"
    fi
    if [ -n "${hostname}" ] ; then
        hostname="hostname        = ${hostname}"
    else
        hostname="#hostname        = ${hostname}"
    fi
    if [ -n "${ssl}" ] ; then
        ssl="ssl             = ${ssl}"
    else
        ssl="#ssl             = ${ssl}"
    fi
    if [ -n "${checkIpSSL}" ] ; then
        checkIpSSL="checkip-ssl     = ${checkIpSSL}"
    else
        checkIpSSL="#checkip-ssl     = ${checkIpSSL}"
    fi
    if [ -n "${checkIpServer}" ] ; then
        checkIpServer="checkip-server  = ${checkIpServer}"
    else
        checkIpServer="#checkip-server  = ${checkIpServer}"
    fi
    if [ -n "${checkIpCommand}" ] ; then
        checkIpCommand="checkip-command = ${checkIpCommand}"
    else
        checkIpCommand="#checkip-command = ${checkIpCommand}"
    fi

    # Remove "http(s)://www." from given provider
    local providerShort=${provider#*//}
    providerShort=${providerShort#www.}

    cat >> ${inadyn_configfile} <<EOF
# ${provider}
provider ${providerPrefix}${providerShort} {
    ${username}
    ${password}
    ${hostname}
    ${ssl}
    ${checkIpSSL}
    ${checkIpServer}
    ${checkIpCommand}
}

EOF
}

# ----------------------------------------------------------------------------
# Create inadyn configuration file
createInadynConfiguration()
{
    checkConfigDir

    # Delete old configuration files if existing
    rm -rf /etc/inadyn/inadyn*.conf
    rm -rf /etc/logrotate.d/inadyn*

    inadyn_configfile='/etc/inadyn/inadyn.conf'

    writeHeader ${inadyn_configfile} ${INADYN_UPDATE_INTERVAL}

    idx=1
    while [ ${idx} -le ${INADYN_ACCOUNT_N} ] ; do
        # Naming of config file

        eval inadyn_account_name='$INADYN_ACCOUNT_'${idx}'_NAME'
        eval inadyn_active='$INADYN_ACCOUNT_'${idx}'_ACTIVE'
        # Write config if account is active
        if [ "$inadyn_active" = 'yes' ] ; then
            eval inadyn_system_type='$INADYN_ACCOUNT_'${idx}'_SYSTEM'
            eval inadyn_use_ssl='$INADYN_ACCOUNT_'${idx}'_USE_SSL'
            eval inadyn_ip_server_name='$INADYN_ACCOUNT_'${idx}'_IP_SERVER'
            eval inadyn_ip_check_command='$INADYN_ACCOUNT_'${idx}'_IP_CHECK_CMD'
            eval user_amount='$INADYN_ACCOUNT_'${idx}'_USER_N'

            if [ "${inadyn_system_type}" = 'dynamic' ] ; then
                providerprefix=''
            else
                providerprefix='default@'
            fi
            if [ "${inadyn_use_ssl}" = 'yes' ] ; then
                useSSL='true'
            else
                useSSL='false'
            fi

            # Reset provider counter
            providerCounter=1

            idx2=1
            while [ ${idx2} -le ${user_amount} ] ; do
                eval inadyn_user='$INADYN_ACCOUNT_'${idx}'_USER_'${idx2}'_NAME'
                eval inadyn_password='$INADYN_ACCOUNT_'${idx}'_USER_'${idx2}'_PASSWORD'
                eval inadyn_alias_number='$INADYN_ACCOUNT_'${idx}'_USER_'${idx2}'_ALIAS_N'
                idx3=1
                while [ ${idx3} -le ${inadyn_alias_number} ] ; do
                    eval current_inadyn_alias='$INADYN_ACCOUNT_'${idx}'_USER_'${idx2}'_ALIAS_'${idx3}

                    if [ ${user_amount} -gt 1 -o ${inadyn_alias_number} -gt 1 ] ; then
                        entryNumber=":${providerCounter}"
                    else
                        entryNumber=''
                    fi

                    writeProviderEntry \
                        "${inadyn_configfile}" \
                        "${providerprefix}" \
                        "${inadyn_system_type}${entryNumber}" \
                        "${inadyn_user}" \
                        "${inadyn_password}" \
                        "${current_inadyn_alias}" \
                        "${useSSL}" \
                        "${useSSL}" \
                        "${inadyn_ip_server_name}" \
                        "${inadyn_ip_check_command}"
                    idx3=$((idx3+1))
                    providerCounter=$((providerCounter+1))
                done
                idx2=$((idx2+1))
            done
        fi
        idx=$((idx+1))
    done
}



# ----------------------------------------------------------------------------
# Check if a mail package exists
checkMailPackage()
{
    if [ -f /etc/check.d/ssmtp -o -f /etc/check.d/vmail ] ; then
        return 0
    fi
    return 1
}



checkConfigDir () {
    if [ ! -d /etc/inadyn/ ] ; then
        mkdir /etc/inadyn
    fi
}


# ----------------------------------------------------------------------------
# Main
if [ "$START_INADYN" = 'yes' ] ; then
    createInadynConfiguration
    rc-update add inadyn
else
    rc-update del inadyn
fi
exit 0
