#!/bin/sh
#------------------------------------------------------------------------------
# eisfair-ng configuration generator script
# Copyright (c) 2007 - 2017 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

retval=0

. /etc/default.d/php-apache2
. /etc/config.d/php-apache2
. /etc/config.d/apache2

APACHE_USER="apache"
PHPv=php7
PHPo=php5


# -----------------------------------------------------------------------------
# load php module if not installed
# -----------------------------------------------------------------------------
load_php_module()
{
    local name="$1"
    apk info -q -e ${PHPv}-$name || apk add -q ${PHPv}-$name
    if [ $? -eq 0 ]; then
        return 0
    else
        # create error message if packages not installed
        logger -p error -t cui-php-apache2 "Fail install: ${PHPv}-$name"
        echo "Fail install: ${PHPv}-$name"
        return 1
    fi
}

# -----------------------------------------------------------------------------
# remove old php module if installed
# -----------------------------------------------------------------------------
remove_php_module()
{
    local name="$1"
    apk del -q ${PHPv}-$name >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        return 0
    else
        # create error message if packages not remove
        logger -p error -t cui-php-apache2 "Fail remove: ${PHPv}-$name"
        return 1
    fi
}

# -----------------------------------------------------------------------------
# update all old php modules
# -----------------------------------------------------------------------------
mlist=""
mkdir -p /etc/${PHPv}/conf.d
if [ -e /etc/${PHPo}/conf.d/ ]
then
    for modfile in /etc/${PHPo}/conf.d/*.ini
    do
        fname=$(basename "$modfile")
        fname=${fname%*.ini}
        case "$fname" in
            eisfair)
                continue
                ;;
            mysql)
                echo -n ""
                ;;
            *)
                mlist="$fname $mlist"
                ;;
        esac
        apk del -q ${PHPo}-$fname
    done
    apk del -f -q ${PHPo}-apache2
    apk add -f -q ${PHPv}
    apk add -f -q ${PHPv}-apache2
    for modfile in $mlist
    do
        load_php_module $modfile
    done
    rm -rf /etc/${PHPo}
else
    apk info -q -e ${PHPv} || apk add -q ${PHPv}
    load_php_module apache2
fi


# -----------------------------------------------------------------------------
# create php info files
# -----------------------------------------------------------------------------
if [ "$PHP_INFO" = "yes" ] ; then
    mkdir -p /var/www/localhost/htdocs
    echo '<?php phpinfo() ?>'>/var/www/localhost/htdocs/info.php
    echo '<?php
    if(!empty($_GET["text"])) {
        Header( "Content-type: image/png");
        // Header( "Content-type: image/jpeg");
        /* create image */
        $image = imagecreate(200,200);
        /* create color R=100, G=0, R=0 */
        $maroon = ImageColorAllocate($image,100,0,0);
        /*  create color R=255, G=255, R=255 */
        $white = ImageColorAllocate($image,255,255,255);
        /*  create color green */
        $green = ImageColorAllocate($image,0,255,0);
        /*  create color cyan */
        $cyan = ImageColorAllocate($image,132,193,255);
        /*  create white background*/
        ImageFilledRectangle($image,0,0,200,200,$white);
        /*  create frame */
        ImageRectangle($image,10,10,190,190,$maroon);
        /*  create a circle */
        imagearc($image, 100, 150, 150, 50, 0, 360, $green);
        /* display font  jv: fix font dir */
        ImageTTFText($image, 45, 10, 30, 100, $cyan, "/usr/share/fonts/TTF/php-apache2-john.ttf",$_GET["text"]);
        /*  render image */
        ImagePNG($image);
        // ImageJPEG($image);
        /* cleanup memory */
        ImageDestroy($image);
    } else {
        function describeGDdyn(){
            echo "<ul>";
            echo "<li>GD support: ";
            if(function_exists("gd_info")){
                echo "<font color=\"#00ff00\">yes</font>";
                $info = gd_info();
                $keys  = array_keys($info);
                for($i=1;$i<count($keys);$i++){
                    echo "</li>\n<li>".$keys[$i] .": " . yesNo($info[$keys[$i]]);
                }
            } else {
                echo "<font color=\"#ff0000\">no</font>";
            }
            echo "</li></ul>";
        }
        function yesNo($bool){
            if($bool){
                return "<font color=\"#00ff00\"> yes</font>";
            }else{
                return "<font color=\"#ff0000\"> no</font>";
            }
        }
        describeGDdyn();
        echo "<p>";
        echo "<form action=gd.php method=GET>";
        echo "<input type=text name=text value=eisfair> <input type=submit value=TestIt!>";
        echo "</form>";
    }
    ?>'>/var/www/localhost/htdocs/gd.php

    chown $APACHE_USER /var/www/localhost/htdocs/info.php /var/www/localhost/htdocs/gd.php
else
    rm -f /var/www/localhost/htdocs/info.php /var/www/localhost/htdocs/gd.php 
fi

# =============================================================================
# Auswerten der Config Parameter
# =============================================================================

# -----------------------------------------------------------------------------
# start
cat >/etc/${PHPv}/conf.d/eisfair.ini <<EOF
[php] 
default_charset = "UTF-8"

EOF

# -----------------------------------------------------------------------------
# Log Error
if [ "$PHP_LOG_ERROR" = "yes" ] ; then
   echo "log_errors = On"  >> /etc/${PHPv}/conf.d/eisfair.ini
else
   echo "log_errors = Off" >> /etc/${PHPv}/conf.d/eisfair.ini
fi

# -----------------------------------------------------------------------------
# Display Errors
if [ "$PHP_DISPLAY_ERRORS" = "yes" ] ; then
    echo "display_errors = On"  >> /etc/${PHPv}/conf.d/eisfair.ini
else
    echo "display_errors = Off" >> /etc/${PHPv}/conf.d/eisfair.ini
fi

# -----------------------------------------------------------------------------
# SendMail Path
if [ -n "$PHP_SENDMAIL_PATH" -a -f "$PHP_SENDMAIL_PATH" ] ; then
    echo "sendmail_path = ${PHP_SENDMAIL_PATH} ${PHP_SENDMAIL_APP}" >>/etc/${PHPv}/conf.d/eisfair.ini 
else
    echo "sendmail_path = sendmail -t -i" >>/etc/${PHPv}/conf.d/eisfair.ini
fi

# -----------------------------------------------------------------------------
# Date Timezone Settings
cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
[Date]
; Defines the default timezone used by the date functions
date.timezone = "$PHP_DATE_TIMEZONE"
;date.default_latitude = 31.766
;date.default_longitude = 35.2333
;date.sunrise_zenith = 90.583333
;date.sunset_zenith = 90.583333

EOF

# -----------------------------------------------------------------------------
# Include Path
[ -n "$PHP_INCLUDE_PATH" ] && PHP_INCLUDE_PATH=":$PHP_INCLUDE_PATH"
[ -e /usr/share/${PHPv}/PEAR ] && pearinc=":/usr/share/${PHPv}/PEAR"
cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
; UNIX: ".:/path1:/path2"
include_path = .:/usr/share/php:/usr/share/${PHPv}${pearinc}${PHP_INCLUDE_PATH}

EOF

# -----------------------------------------------------------------------------
# Upload directory
if [ -d "$PHP_UPLOAD_DIR" ] ; then
    echo "upload_tmp_dir = $PHP_UPLOAD_DIR" >> /etc/${PHPv}/conf.d/eisfair.ini
else
    echo 'upload_tmp_dir = "/tmp"' >> /etc/${PHPv}/conf.d/eisfair.ini
fi

# -----------------------------------------------------------------------------
# file and memory limit
cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
; Maximum allowed size for uploaded files.
upload_max_filesize = $PHP_MAX_UPLOAD_FILESIZE
; Maximum execution time of each script, in seconds
max_execution_time = $PHP_MAX_EXECUTION_TIME
; Maximum amount of memory a script may consume (8MB)
memory_limit = $PHP_MEMORY_LIMIT
; Maximum size of POST data that PHP will accept.
post_max_size = $PHP_MAX_POST_SIZE

EOF


# -----------------------------------------------------------------------------
# MYSQL
if [ "$PHP_EXT_MYSQL" = "yes" ] ; then
    if [ -z "$PHP_EXT_MYSQL_SOCKET" -a -z "$PHP_EXT_MYSQL_HOST" ] ; then
        [ -e "/run/mysqld/mysqld.sock" ] && PHP_EXT_MYSQL_SOCKET="/run/mysqld/mysqld.sock"
    fi
    if [ -z "$PHP_EXT_MYSQL_HOST" ] ; then
        PHP_EXT_MYSQL_PORT=""
    else
        [ -z "$PHP_EXT_MYSQL_PORT" ] && PHP_EXT_MYSQL_PORT="3306"
    fi

    load_php_module pdo_mysql
    if [ $? -eq 0 ]; then
        cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
[pdo_mysql]
pdo_mysql.cache_size=2000
pdo_mysql.default_socket=${PHP_EXT_MYSQL_SOCKET}

EOF
    fi

    load_php_module mysqli
    if [ $? -eq 0 ]; then
        cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
[mysqli]
mysqli.max_persistent=-1
mysqli.allow_local_infile=On
mysqli.allow_persistent=On
mysqli.max_links=-1
mysqli.cache_size=2000
mysqli.default_port=${PHP_EXT_MYSQL_PORT}
mysqli.default_socket=${PHP_EXT_MYSQL_SOCKET}
mysqli.default_host=${PHP_EXT_MYSQL_HOST}
mysqli.default_user=
mysqli.default_pw=
mysqli.reconnect=Off

[mysqlnd]
mysqlnd.collect_statistics=On
mysqlnd.collect_memory_statistics=Off
;mysqlnd.net_cmd_buffer_size=2048
;mysqlnd.net_read_buffer_size=32768

EOF
    fi
else
    remove_php_module pdo_mysql
    remove_php_module mysqli
fi

# ------------------------------------------------------------------------------
# INTERBASE
if [ "$PHP_EXT_INTER" = "yes" ] ; then
#    load_php_module interbase
    if [ $? -eq 0 ]; then
        cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
[interbase]
; Allow or prevent persistent links.
ibase.allow_persistent = 1
; Maximum number of persistent links.  -1 means no limit.
ibase.max_persistent = -1
; Maximum number of links (persistent + non-persistent).  -1 means no limit.
ibase.max_links = -1
; Default database name for ibase_connect().
;ibase.default_db =
; Default username for ibase_connect().
;ibase.default_user =
; Default password for ibase_connect().
;ibase.default_password =
; Default charset for ibase_connect().
;ibase.default_charset =
; Default timestamp format.
ibase.timestampformat = "%Y-%m-%d %H:%M:%S"
; Default date format.
ibase.dateformat = "%Y-%m-%d"
; Default time format.
ibase.timeformat = "%H:%M:%S"
EOF
    fi
else
    remove_php_module interbase
fi

# -----------------------------------------------------------------------------
# MSSQL
if [ "$PHP_EXT_MSSQL" = "yes" ] ; then
    load_php_module mssql
    if [ $? -eq 0 ]; then
        cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
[mssql]
; Allow or prevent persistent links.
mssql.allow_persistent = On
; Maximum number of persistent links.  -1 means no limit.
mssql.max_persistent = -1
; Maximum number of links (persistent+non persistent).  -1 means no limit.
mssql.max_links = -1
; Minimum error severity to display.
mssql.min_error_severity = 10
; Minimum message severity to display.
mssql.min_message_severity = 10
; Compatibility mode with old versions of PHP 3.0.
mssql.compatability_mode = Off
; Connect timeout
;mssql.connect_timeout = 5
; Query timeout
;mssql.timeout = 60
; Valid range 0 - 2147483647.  Default = 4096.
;mssql.textlimit = 4096
; Valid range 0 - 2147483647.  Default = 4096.
;mssql.textsize = 4096
; Limits the number of records in each batch.  0 = all records in one batch.
;mssql.batchsize = 0
; Specify how datetime and datetim4 columns are returned
; On => Returns data converted to SQL server settings
; Off => Returns values as YYYY-MM-DD hh:mm:ss
;mssql.datetimeconvert = On
; Use NT authentication when connecting to the server
mssql.secure_connection = Off
; Specify max number of processes. -1 = library default
; msdlib defaults to 25
; FreeTDS defaults to 4096
;mssql.max_procs = -1
; Specify client character set.
; If empty or not set the client charset from freetds.comf is used
; This is only used when compiled with FreeTDS
;mssql.charset = "ISO-8859-1"

EOF
    fi
else
    remove_php_module mssql
fi

# -----------------------------------------------------------------------------
# POSTGRESQL
if [ "${PHP_EXT_PGSQL}" = "yes" ] ; then
    load_php_module pgsql
    if [ $? -eq 0 ]; then
        cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
[PostgresSQL]
; Allow or prevent persistent links.
; http://php.net/pgsql.allow-persistent
pgsql.allow_persistent = On
; Detect broken persistent links always with pg_pconnect().
; Auto reset feature requires a little overheads.
; http://php.net/pgsql.auto-reset-persistent
pgsql.auto_reset_persistent = Off
; Maximum number of persistent links.  -1 means no limit.
; http://php.net/pgsql.max-persistent
pgsql.max_persistent = -1
; Maximum number of links (persistent+non persistent).  -1 means no limit.
; http://php.net/pgsql.max-links
pgsql.max_links = -1
; Ignore PostgreSQL backends Notice message or not.
; Notice message logging require a little overheads.
; http://php.net/pgsql.ignore-notice
pgsql.ignore_notice = 0
; Log PostgreSQL backends Notice message or not.
; Unless pgsql.ignore_notice=0, module cannot log notice message.
; http://php.net/pgsql.log-notice
pgsql.log_notice = 0

EOF
    fi
    load_php_module pdo_pgsql
else
    remove_php_module pqsql
    remove_php_module pdo_pgsql   
fi

# -----------------------------------------------------------------------------
# SQLite3
if [ "$PHP_EXT_SQLITE3" = "yes" ] ; then
    load_php_module sqlite3
    load_php_module pdo_sqlite
else
    remove_php_module sqlite3
    remove_php_module pdo_sqlite
fi

# -----------------------------------------------------------------------------
# CTYPE
if [ "$PHP_EXT_CTYPE" = "yes" ] ; then
    load_php_module ctype
else
    remove_php_module ctype
fi

# -----------------------------------------------------------------------------
# SOAP
if [ "$PHP_EXT_SOAP" = "yes" ] ; then
    load_php_module soap
else
    remove_php_module soap    
fi

# -----------------------------------------------------------------------------
# GD
if [ "$PHP_EXT_GD" = "yes" ] ; then
    load_php_module gd
else
    remove_php_module gd
fi

# -----------------------------------------------------------------------------
# LDAP
if [ "$PHP_EXT_LDAP" = "yes" ] ; then
    load_php_module ldap
    if [ $? -eq 0 ]; then
        cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
[ldap]
; Sets the maximum number of open links or -1 for unlimited.
ldap.max_links = -1

EOF
    fi
else
    remove_php_module ldap    
fi

# -----------------------------------------------------------------------------
# CURL
if [ "$PHP_EXT_CURL" = "yes" ] ; then
    load_php_module curl
else
    remove_php_module curl    
fi

# -----------------------------------------------------------------------------
# CACHE
rm -f /etc/${PHPv}/conf.d/apc.ini
rm -f /etc/${PHPv}/conf.d/apcu.ini
rm -f /etc/${PHPv}/conf.d/xcache.ini
rm -f /etc/${PHPv}/conf.d/memcache.ini
rm -f /etc/${PHPv}/conf.d/opcache.ini

if [ "$PHP_EXT_CACHE" = "apc" ] ; then
    load_php_module apcu
elif [ "$PHP_EXT_CACHE" = "xcache" ] ; then
    load_php_module xcache
    if [ $? -eq 0 ]; then
        cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
xcache.size=64M
xcache.var_size=64M

EOF
    fi
elif [ "$PHP_EXT_CACHE" = "memcache" ] ; then
    load_php_module memcache
    if [ $? -eq 0 ]; then
        cat >>/etc/${PHPv}/conf.d/eisfair.ini <<EOF
;memcache.allow_failover="1"
;memcache.max_failover_attempts="20"
;memcache.chunk_size="8192"
;memcache.default_port="11211"
;memcache.hash_strategy="standard"
;memcache.hash_function="crc32"
;session.save_handler="files"
;session.save_path=""
;memcache.protocol=ascii
;memcache.redundancy=1
;memcache.session_redundancy=2
;memcache.compress_threshold=20000
;memcache.lock_timeout=15

EOF
    fi
elif [ "$PHP_EXT_CACHE" = "opcache" ] ; then
    load_php_module opcache
fi

# -----------------------------------------------------------------------------
# JSON
if [ "$PHP_EXT_JSON" = "yes" ] ; then
    load_php_module json
else
    remove_php_module json   
fi

# -----------------------------------------------------------------------------
# GETTEXT
if [ "$PHP_EXT_GETTEXT" = "yes" ] ; then
    load_php_module gettext
else
    remove_php_module gettext
fi

# -----------------------------------------------------------------------------
# ICONV
if [ "$PHP_EXT_ICONV" = "yes" ] ; then
    load_php_module iconv
else
    remove_php_module iconf    
fi

# -----------------------------------------------------------------------------
# IMAP
if [ "$PHP_EXT_IMAP" = "yes" ] ; then
    load_php_module imap
else
    remove_php_module imap    
fi

# -----------------------------------------------------------------------------
# OPENSSL
if [ "$PHP_EXT_SSL" = "yes" ] ; then
    load_php_module openssl
else
    remove_php_module openssl   
fi

# -----------------------------------------------------------------------------
# XML
if [ "$PHP_EXT_XML" = "yes" ] ; then
    load_php_module xml
else
    remove_php_module xml
fi
# -----------------------------------------------------------------------------
# XMLREADER
if [ "$PHP_EXT_XML" = "yes" ] ; then
    load_php_module xmlreader
else
    remove_php_module xmlreader
fi

# -----------------------------------------------------------------------------
# XMLWRITER
if [ "$PHP_EXT_XML" = "yes" ] ; then
    load_php_module xmlwriter
else
    remove_php_module xmlwriter
fi

# -----------------------------------------------------------------------------
# ZIP
if [ "$PHP_EXT_ZIP" = "yes" ] ; then
    load_php_module zip
else
    remove_php_module zip    
fi

# -----------------------------------------------------------------------------
# ZLIB
if [ "$PHP_EXT_ZLIB" = "yes" ] ; then
    load_php_module zlib
else
    remove_php_module zlip  

fi


# =============================================================================
# Restart apache
[ "$START_APACHE2" = "yes" ] && rc-service -i apache2 restart 

exit 0
