/* ---------------------------------------------------------------------
 * File: textdlg.c
 * (multiline text input dialog)
 *
 * Copyright (C) 2017
 * Daniel Vogel, <daniel_vogel@t-online.de>
 *
 * Last Update:  $Id$
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * ---------------------------------------------------------------------
 */

#include "global.h"
#include "textdlg.h"

#define IDC_EDTEXT     10

/* ---------------------------------------------------------------------
 * TextDlgButtonHook
 * Handle button events
 * ---------------------------------------------------------------------
 */
static void
TextDlgButtonHook(void* w, void* c)
{
	CUIWINDOW   *win  = (CUIWINDOW*) w;
	CUIWINDOW   *ctrl = (CUIWINDOW*) c;
	TEXTDLGDATA *data = (TEXTDLGDATA*) win->InstData;

	if (ctrl->Id == IDOK)
	{
		ctrl = WindowGetCtrl(win, IDC_EDTEXT);
		if (ctrl)
		{
			MemoGetText(ctrl, data->Text, MAX_ML_INPUT_SIZE);
		}
		WindowClose(win, IDOK);
	}
	else
	{
		WindowClose(win, IDCANCEL);
	}
}

/* ---------------------------------------------------------------------
 * TextDlgCreateHook
 * Create dialog controls
 * ---------------------------------------------------------------------
 */
static void
TextDlgCreateHook(void* w)
{
	CUIWINDOW   *win = (CUIWINDOW*) w;
	CUIWINDOW   *ctrl;
	TEXTDLGDATA *data = (TEXTDLGDATA*) win->InstData;

	ctrl = MemoNew(win, _T(""), 1, 1, 76, 15, IDC_EDTEXT, CWS_NONE, CWS_NONE);
	MemoSetText(ctrl, data->Text);
	WindowSetFocus(ctrl);
	WindowCreate(ctrl);

	ctrl = ButtonNew(win, _T("&OK"), 23, 17, 10, 1, IDOK, CWS_DEFOK, CWS_NONE);
	ButtonSetClickedHook(ctrl, TextDlgButtonHook, win);
	WindowCreate(ctrl);

	ctrl = ButtonNew(win, _T("&Cancel"), 38, 17, 10, 1, IDCANCEL, CWS_DEFCANCEL, CWS_NONE);
	ButtonSetClickedHook(ctrl, TextDlgButtonHook, win);
	WindowCreate(ctrl);
}

/* ---------------------------------------------------------------------
 * TextDlgDestroyHook
 * Destroy dialog data
 * ---------------------------------------------------------------------
 */
static void
TextDlgDestroyHook(void* w)
{
	CUIWINDOW* win = (CUIWINDOW*) w;
	free(win->InstData);
}

/* ---------------------------------------------------------------------
 * TextDlgKeyHook
 * Handle EVENT_KEY events
 * ---------------------------------------------------------------------
 */
static int
TextDlgKeyHook(void* w, int key)
{
	CUIWINDOW* win = (CUIWINDOW*) w;

	if (win->IsEnabled)
	{
		if (key == KEY_F0 + 8)
		{
			// delete line!
			CUIWINDOW   *ctrl;
			ctrl = WindowGetCtrl(win, IDC_EDTEXT);
			if (ctrl)
			{
				MemoDeleteLine(ctrl);
			}
			return TRUE;
		}
	}
	return FALSE;
}

/* ---------------------------------------------------------------------
 * TextDlgNew
 * Create a new dialog instance
 * ---------------------------------------------------------------------
 */
CUIWINDOW*
TextDlgNew(CUIWINDOW* parent, const wchar_t* title, int sflags, int cflags)
{
	if (parent)
	{
		CUIWINDOW* dlg;
		int flags = sflags | CWS_POPUP | CWS_BORDER | CWS_CENTERED;
		flags &= ~(cflags);

		dlg = WindowNew(parent, 0, 0, 80, 21, flags);
		dlg->Class = _T("INPUT_DLG");
		WindowColScheme(dlg, _T("DIALOG"));
		WindowSetCreateHook(dlg, TextDlgCreateHook);
		WindowSetDestroyHook(dlg, TextDlgDestroyHook);
		WindowSetKeyHook(dlg, TextDlgKeyHook);

		dlg->InstData = (TEXTDLGDATA*) malloc(sizeof(TEXTDLGDATA));
		((TEXTDLGDATA*)dlg->InstData)->Text[0] = 0;

		WindowSetText(dlg, title);
		return dlg;
	}
	return NULL;
}

/* ---------------------------------------------------------------------
 * TextDlgGetData
 * Get a reference to the dialog data
 * ---------------------------------------------------------------------
 */
TEXTDLGDATA*
TextDlgGetData(CUIWINDOW* win)
{
	if (win && (wcscmp(win->Class, _T("INPUT_DLG")) == 0))
	{
		return (TEXTDLGDATA*) win->InstData;
	}
	return NULL;
}

