#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/base.sh - apply configuration for base
# Copyright (c) 2001-2017 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:/usr/sbin
# include base-config
. /etc/config.d/base

# include alpine base config
. /etc/rc.conf

#----------------------------------------------------------------------------
# force unicode!
#----------------------------------------------------------------------------
sed -i 's/^#unicode=.*/unicode="YES"/' /etc/rc.conf
sed -i 's/^unicode=.*/unicode="YES"/'  /etc/rc.conf

#----------------------------------------------------------------------------
# Set time zone
#----------------------------------------------------------------------------
case "$TIME_ZONE" in
    utc|UTC) 
	echo "UTC" > /etc/TZ 
	TIME_ZONE="UTC"
	zdir=""
	;;
    Europe*) 
	echo "CET-1CEST,M3.5.0,M10.5.0/3" > /etc/TZ 
	zdir="${TIME_ZONE%/*}"
	;;
    *) 
        echo "$TIME_ZONE" > /etc/TZ
	zdir="${TIME_ZONE%/*}"
	;;
esac

if [ -f "/usr/share/zoneinfo/$TIME_ZONE" ]
then
    rm -r /etc/zoneinfo
    mkdir -p /etc/zoneinfo/$zdir
    cp "/usr/share/zoneinfo/$TIME_ZONE" "/etc/zoneinfo/$zdir"
    rm -f /etc/localtime
    ln -s /etc/zoneinfo/$TIME_ZONE /etc/localtime
fi

# set extern timesync for raspi
if grep -q armhf /etc/apk/arch
then
    if [ -z "$(lsmod | grep i2c_bcm2708)" ]
    then
	apk info -q -e chrony || apk add --quiet chrony
	rc-update add chronyd default
    fi
fi

#----------------------------------------------------------------------------
# config and restart network (if not work with lxc container 
#----------------------------------------------------------------------------
case "$rc_sys" in
    lxc)
	echo "Container - no network setup"
	;;
    *) 
	/var/install/config.d/base-net.sh
	;;
esac

#----------------------------------------------------------------------------
# create logrotate configuration
#----------------------------------------------------------------------------
if [ "$START_SYSLOG" = yes ]
then
    # remove old additional syslog entries
    for oldentry in source destination log filter
    do
        rm -f /etc/syslog-ng/syslog-ng-$oldentry.base
    done
    cat <<-EOF >/etc/logrotate.d/syslog
	#-----------------------------------------------------------------
	# /etc/logrotate.d/syslog
	# This file has been created automatically by Eisfair system setup
	#-----------------------------------------------------------------
	EOF
    # add logrotate entries without reload syslog-ng command first
    idx=1
    while [ $idx -le 0$SYSLOG_DEST_N ]
    do
        logmsgname="baselog$idx"
        eval logtarget='$SYSLOG_DEST_'$idx'_TARGET'
        eval logfilter='$SYSLOG_DEST_'$idx'_FILTER'
        case "$logtarget" in
            # Syslog to external IP address
            1*|2*|3*|4*|5*|6*|7*|8*|9*)
                logtarget=`echo "$logtarget" | sed 's#@##'`
                echo "  destination df_$logmsgname { udp(\"$logtarget\"); };" >>/etc/syslog-ng/syslog-ng-destination.base
                ;;
            # Syslog to file
            /*)
                echo "  destination df_$logmsgname { file(\"$logtarget\"); };" >>/etc/syslog-ng/syslog-ng-destination.base
                # add logrotate entry
                eval loginterval='$SYSLOG_DEST_'$idx'_INTERVAL'
                eval logmaxcount='$SYSLOG_DEST_'$idx'_MAXCOUNT'
                cat <<-EOF >>/etc/logrotate.d/syslog
				${logtarget} {
				  rotate $logmaxcount
				  $loginterval
				  missingok
				  notifempty
				  compress
				}

				EOF
                ;;
        esac
		[ -n "$logfilter" ] && echo "  filter f_$logmsgname { $logfilter; };" >>/etc/syslog-ng/syslog-ng-filter.base
		{
		echo -e "log {\n        source(s_all);"
		[ -n "$logfilter" ] && echo "        filter(f_$logmsgname);"
		echo -e "        destination(df_$logmsgname);\n};\n"
		} >> /etc/syslog-ng/syslog-ng-log.base
		idx=$((idx+1))
	done

	echo_rotate_entry () {
		local maxcount="$1" interval="$2"
		shift 2
		cat <<-EOF
		$@ {
		  rotate $maxcount
		  $interval
		  missingok
		  notifempty
		  create 0644
		  compress
		EOF
	}

	echo_postrotate_entry () {
		cat <<-EOF
		  postrotate
		    /etc/init.d/syslog-ng reload >/dev/null
		  endscript
		EOF
	}

	echo_rotate_end () {
		echo -e "}\n"
	}
	{
	echo_rotate_entry $SYSLOG_AUTH_MAXCOUNT $SYSLOG_AUTH_INTERVAL /var/log/auth.log
	[ "$SYSLOG_AUTH_RELOAD" = yes ] && echo_postrotate_entry
	echo_rotate_end
	echo_rotate_entry $SYSLOG_MAIL_MAXCOUNT $SYSLOG_MAIL_INTERVAL /var/log/mail.log /var/log/mail.info /var/log/mail.warn /var/log/mail.err
	[ "$SYSLOG_MAIL_RELOAD" = yes ] && echo_postrotate_entry
	echo_rotate_end
	echo_rotate_entry $SYSLOG_KERNEL_MAXCOUNT $SYSLOG_KERNEL_INTERVAL /var/log/kern.log
	[ "$SYSLOG_KERNEL_RELOAD" = yes ] && echo_postrotate_entry
	echo_rotate_end
	echo_rotate_entry $SYSLOG_MESSAGES_MAXCOUNT $SYSLOG_MESSAGES_INTERVAL /var/log/error.log
	[ "$SYSLOG_MESSAGES_RELOAD" = yes ] && echo_postrotate_entry
	echo_rotate_end
	} >> /etc/logrotate.d/syslog
	[ "$SYSLOG_SOURCE_UDP" = yes ] && echo "  udp( ip(0.0.0.0) port(514) );" >/etc/syslog-ng/syslog-ng-source.base
	/etc/init.d/syslog stop >/dev/null 2>&1
	/sbin/rc-update -q del syslog boot >/dev/null 2>&1
	/sbin/rc-update -q add syslog-ng boot >/dev/null 2>&1
	/etc/init.d/syslog-ng update
	if [ -e /var/run/syslog-ng.pid ]
	then
		/etc/init.d/syslog-ng reload
	else
		/etc/init.d/syslog-ng start
	fi
else
	/etc/init.d/syslog-ng stop
	/sbin/rc-update del syslog-ng boot
	/sbin/rc-update add syslog boot
	/etc/init.d/syslog start
fi


#----------------------------------------------------------------------------
# keyboard and console setup
#----------------------------------------------------------------------------
mkdir -p /etc/keymap
[ -f /etc/conf.d/keymaps ] && sed -i '/^KEYMAP=/d' /etc/conf.d/keymaps
if [ "$KEYMAP" <> "en" ]
then
    rm -f /etc/keymap/*
    if [ -d /lib/kbd/keymaps ]
    then
	[ -e "/lib/kbd/keymaps/$KEYMAP.bmap.gz" ] || KEYMAP="de"
	cp -f "/lib/kbd/keymaps/$KEYMAP.bmap.gz" /etc/keymap/$KEYMAP.bmap.gz
	echo "KEYMAP=/etc/keymap/$KEYMAP.bmap.gz" >> /etc/conf.d/keymaps
	# we actually load the keymap now
	zcat /etc/keymap/$KEYMAP.bmap.gz | loadkmap
    fi
    rc-update -q add keymaps boot
else
    rc-update -q del keymaps boot
fi

if [ -e /lib/kbd/consolefonts ]
then
    # add file extension to font basename:
    cd /lib/kbd/consolefonts
    for I in $CONSOLEFONT.* ; do
	    CONSOLEFONT="$I"
    done
    sed -i "s/^consolefont=.*/consolefont=\"$CONSOLEFONT\"/" /etc/conf.d/consolefont
    #sed -i '/^consoletranslation=/d' /etc/conf.d/consolefont
    #sed -i '/^unicodemap=/d' /etc/conf.d/consolefont
    /sbin/rc-service -q consolefont restart >/dev/null 2>&1
fi
[ -f /etc/init.d/kbd-mini ] && /sbin/rc-service -q kbd-mini start >/dev/null 2>&1

# Set console blank time ESC 9 and VESA powerdown ESC 14
if [ "0$CONSOLE_BLANK_TIME" -eq 0 ]
then
    echo -n -e '\033[9;0]\033[14;0]' >/dev/console
else
    echo -n -e "\033[9;${CONSOLE_BLANK_TIME}]\033[14;${CONSOLE_BLANK_TIME}]" >/dev/console
fi


exit 0
