#!/bin/bash
###########

mailfile=$1

OLDIFS=$IFS
IFS=

cleanUp() {
    IFS=$OLDIFS
    rm -fr $tmp_path
}

# unzip zip files and find executable
unpack() {
    local file=""
    for file in $@
    do
        if [ "${file%.[zZ][iI][pP]}" != "${file}" ]
        then
            unzip -o "$file" >/dev/null
            rm "${file}"
            unpack *
        elif [ "${file%.[rR][aA][rR]}" != "${file}" ]
        then
            unrar e "$file" >/dev/null
            rm "${file}"
            unpack *
        elif [ -d "${file}" ]
        then
            unpack ${file}/*
            rm -rf ${file}
        elif [ "${file%.[eE][xX][eE]}" != "${file}" ]
        then
            echo "SUB: executable included"
            exit 10
        elif [ "${file%.[cC][oO][mM]}" != "${file}" ]
        then
            echo "SUB: executable included"
            exit 10
        elif [ "${file%.[bB][aA][tT]}" != "${file}" ]
        then
            echo "SUB: executable included"
            exit 10
        elif [ "${file%.[cC][mM][dD]}" != "${file}" ]
        then
            echo "SUB: executable included"
            exit 10
        elif [ "${file%.[dD][oO][cC][mM]}" != "${file}" ]
        then
            echo "SUB: macro"
            exit 10
        elif [ "${file%.[hH][tT][aA]}" != "${file}" ]
        then
            echo "SUB: executable included"
            exit 10
        elif [ "${file%.[pP][iI][fF]}" != "${file}" ]
        then
            echo "SUB: executable included"
            exit 10
        elif [ "${file%.[sS][cC][fFrRtT]}" != "${file}" ]
        then
            echo "SUB: executable included"
            exit 10
        elif [ "${file%.[jJ][sS]}" != "${file}" ]
        then
            echo "SUB: java script"
            exit 10
        elif [ "${file%.[jJ][sS][eE]}" != "${file}" ]
        then
            echo "SUB: java script"
            exit 10
        elif [ "${file%.[vV][bB][sS]}" != "${file}" ]
        then
            echo "SUB: vbs"
            exit 10
        elif [ "${file%.[dDxX][oOlL][cCsS]}" != "${file}" ]
        then
            if grep -qEi "regedit|autoopen|autoexec" "$file"
            then
                echo "SUB: document with macro"
                exit 10
            fi
        elif [ "${file%.[pP][dD][fF]}" != "${file}" ]
        then
            if grep -qEi "regedit|autoopen|autoexec" "$file"
            then
                echo "SUB: pdf with macro"
                exit 10
            fi
        elif [ "${file%.[jJ][pP][gG]}" != "${file}" ]
        then
            # picture not scann
            echo -n ""
        else
            if grep -q "http:\/\/" "$file"
            then
                if grep -qEi "bankbot|bot-program" "$file".
                then
                    echo "SUB: bankbot"
                    exit 10
                fi
            fi
        fi

    done
}

trap cleanUp EXIT

tmp_path=/var/tmp/smc-$$
mkdir -p -m0777 "$tmp_path"
cd "$tmp_path"

/usr/bin/ripmime --unique_names -d "$tmp_path" -i "$mailfile"

unpack *

# ------------------------------------------------------------------------------
# Example for use return values:
# BCC: add recipient
# XHD: X-Header entry
# SUB: add subject prefix
# picture only spam - add e-mail subject prefix "Spam:"
if grep -qE "Received:.*\.eu \[" "$mailfile"
then
    if grep -qE "Content-Type: image" "$mailfile"
    then
        echo "SUB: Spam:"
        exit 0    # view only exit = 0
#        exit 9    # activate with exit = 9
    fi
fi
# send sys error message Bcc: mailadmin
if grep -qEi "system error|critical system state" $mailfile ; then
    if ! grep -q "mailadmin@domain.lan" $mailfile ; then
        echo "BCC: <mailadmin@domain.lan>"
        echo "XHD: SYSCOPY"
        exit 0    # view only exit = 0
#        exit 9    # activate with exit = 9
    fi
fi

exit 0
