/*
 * smc-milter.c
 * Description:     SMC anti-spam e-mail filter
 * Copyright (c) 2003-2008 Aleksander Djuric.
 *
 * jens@eisfair.org
 * remove autospf 
 * remove classicspf 
 * remove greylisting 
 * remove autoswl 
 * add antivirus operations
 * add signature from mysql table  
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <assert.h>
#include <syslog.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <pthread.h>
#include <pwd.h>
#include <netinet/in.h>
#include <spf2/spf.h>
/*#include <arpa/nameser.h>*/
#include <arpa/inet.h>
/*#include <resolv.h>*/
#include <netdb.h>
#include <mysql/mysql.h>
#include <libmilter/mfapi.h>
#include "smc-milter.h"
#include "utils.h"
#include "config.h"
#include "virus.h"
#include "iptab.h"

#if !defined O_SYNC && defined O_FSYNC
#define O_SYNC O_FSYNC
#endif

/* Static variables for config defaults, etc. */
static char *smfisock  = NULL;
static char *smfiuser  = NULL;
static char *dbhost    = NULL;
static int   dbport    = 0;
static char *dbname    = NULL;
static char *dbtable   = NULL;
static char *dbuser    = NULL;
static char *dbpass    = NULL;
static char *optremove = NULL;
static char *quarantinedir = NULL;
static char *avmail    = NULL;
static int  clamcheck  = 0;
static char *clamsock  = NULL;
static int  fprotcheck = 0;
static int  signatureadd = 0;
static int  spfcheck = 0;
static int  debuglevel = 0;
static char *scriptfile = NULL;
static iptab whitelist;

struct mlfiPriv {
    unsigned long status;
    long body_offset;                     /* offset from begin of file to data section */
    char helo_host[MAXLINE];
    char conn_addr[INET_ADDRSTRLEN];
    char from_addr[MAXLINE];
    char report[MAXLINE];
    char temp_mail[TMP_FILE_SIZE];        /* mail file complete, for virus scann! */
    char temp_signtxt[TMP_FILE_SIZE];     /* temp signature text file */
    char temp_signhtm[TMP_FILE_SIZE];     /* temp signature html file */
    char *subject;                        /* Original subject */
    char *addbcc;                         /* New BCC entry */
    char *newsubject;                     /* Set new subject */
    char *newhead;                        /* Set new X-Header */
    int  utf8;                            /* if found "charset=UTF-8" set to 1 */
    int child1pid;                        /* pid from virus scanner script */
    int child2pid;                        /* pid from body script */
    int whitelisted;                      /* white list check */
};


/* ----------------------------------------------------------------------------
 * Store the name of the virus in the subject of the e-mail
 * ----------------------------------------------------------------------------*/
static void
setnewsubject(SMFICTX *ctx, const char *virusname) {
    struct mlfiPriv *priv = MLFIPRIV;
    char buffer[MAXLINE];

    memset(buffer, '\0', MAXLINE );
    if(priv->subject != NULL) {
        smfi_addheader(ctx, "X-Original-Subject", priv->subject);
        snprintf(buffer, MAXLINE - 1, "%s %s", virusname, priv->subject);
        smfi_chgheader(ctx, "Subject", 1, buffer);
    } else {
        snprintf(buffer, MAXLINE - 1, "%s", virusname);
        smfi_addheader(ctx, "Subject", buffer);
    }
}


/* ----------------------------------------------------------------------------
 * replace original email body with new createt body 
 * ----------------------------------------------------------------------------*/
int
change_body(SMFICTX *ctx) {
    int errval, rc1, rc2 = 0;
    int fd = -1;
    ssize_t nread;
    unsigned char buffer[MAXLINE];
    struct mlfiPriv *priv = MLFIPRIV;

    if ((fd = open(priv->temp_mail, O_RDONLY)) < 0) {
      syslog(LOG_ERR, "can't open %s : %s", priv->temp_mail, strerror(errno));
      return -1;
    }
    /* go to the body begin */
    if( lseek( fd, priv->body_offset, SEEK_SET ) < 0 ) {
      syslog(LOG_ERR, "cannot seek to offset=%ld for file %s",priv->body_offset, priv->temp_mail );
      (void) close(fd); 
      return -1;
    }
    while( (rc1 = file_read( fd, buffer, sizeof(buffer), FILEIOTIMEOUT, &nread, &errval )) == 0 && nread > 0 ) {
      /* replace the message body */
      if( (rc2 = smfi_replacebody( ctx, (unsigned char*)buffer, nread )) != MI_SUCCESS ) break;
    }
    (void) close(fd);
    if (rc1 != 0) 
      syslog(LOG_ERR, "change_body: Error when reading message body, %s", strerror(errval));
    if (rc2 != 0) 
      syslog(LOG_ERR, "change_body: Error when replacing the original message body");
    syslog(LOG_INFO, "Body modified: %s", priv->temp_mail);
    return 0;
}


/* ----------------------------------------------------------------------------
 * add signature or disclaimer from mysql table
 * ----------------------------------------------------------------------------*/
bool
add_external_signature(SMFICTX *ctx ) {
    struct mlfiPriv *priv = MLFIPRIV;
    char buffer[MAXLINE]; 
    MYSQL_RES *result;
    MYSQL_ROW row;
    MYSQL *g_mysql;
    unsigned long *lengths;
    int row_count;
    int count = 0;
    int fd;
    FILE *fp;

    if ( file_signature_content( priv->temp_mail ) == 1 ){
      syslog(LOG_INFO, "Signatur found %s", priv->temp_mail );
      return FALSE;
    }
    snprintf(buffer, 1023, "SELECT signature FROM %s WHERE email = '%s' ORDER BY email LIMIT 1", dbtable, priv->from_addr );

    g_mysql= mysql_init( NULL );
    mysql_options(g_mysql, MYSQL_SET_CHARSET_NAME, "utf8");
    mysql_options(g_mysql, MYSQL_INIT_COMMAND, "SET NAMES utf8");

    if ( ! mysql_real_connect(g_mysql, dbhost, dbuser, dbpass, dbname, dbport, 0, 0)) {
      syslog(LOG_ERR, "add_external_signature: mysql_real_connect failed");
      syslog(LOG_ERR, "%s", mysql_error(g_mysql));
      mysql_close(g_mysql);
      return FALSE;
    } else  {
      /* Reconnect must be set *after* real_connect()! */
      mysql_options(g_mysql, MYSQL_OPT_RECONNECT, "1"); 
      if (debuglevel > 2)
        syslog(LOG_INFO, "mysql: new connect" );    
    }

    while ( count < 4) {
      /* check the mysql connect */
      if (debuglevel > 2)
        syslog(LOG_INFO, "mysql_query: %s", buffer ); 
      if (mysql_query(g_mysql, buffer ) == 0) {
        /* get the results */
        result = mysql_store_result(g_mysql);
        row_count = mysql_num_rows(result);
        if (row_count) {
          if (*priv->temp_signtxt == '\0') {
            strcpy(priv->temp_signtxt, TMP_SIGNTXT);
            fd = mkstemp(priv->temp_signtxt);
            if (fd == -1)
              syslog(LOG_ERR, "mkstemp: can't create: %s", strerror(errno));
            else {
              fchmod(fd, 0644);
              (void) close( fd );
            }
              }
          if (*priv->temp_signhtm == '\0') {
            strcpy(priv->temp_signhtm, TMP_SIGNHTM);
            fd = mkstemp(priv->temp_signhtm);
            if (fd == -1)
              syslog(LOG_ERR, "mkstemp: can't create: %s", strerror(errno));
            else {
              fchmod(fd, 0644);
              (void) close( fd );
            }
          }
          fp = fopen(priv->temp_signtxt, "wb");
          row = mysql_fetch_row(result);
          lengths = mysql_fetch_lengths(result);
          if ( priv->utf8 == 0) {
            /* if mail not utf-8 encoded then translate signature to ISO */
            char *tmp1 = UTF8toISO(row[0], lengths[0]);
            if (tmp1) {
              fwrite(tmp1, strlen(tmp1), 1, fp);
              SAFE_FREE(tmp1);
            } else {
              fwrite(row[0], lengths[0], 1, fp);
            }
          } else {
            fwrite(row[0], lengths[0], 1, fp);
          }
          fwrite(STR_CRLF, strlen(STR_CRLF), 1, fp);
          fclose(fp);

          fp = fopen(priv->temp_signhtm, "wb");
          fwrite( MOZ_SIGNATURE, strlen(MOZ_SIGNATURE), 1, fp);
          char *tmp2 = UTF8toHTML(row[0], lengths[0]);
          if (tmp2) {
            fwrite(tmp2, strlen(tmp2), 1, fp);
            SAFE_FREE(tmp2);
          } else
            fwrite(row[0], lengths[0], 1, fp);
          fwrite("</pre>", 6, 1, fp);                
          fwrite(STR_CRLF, strlen(STR_CRLF), 1, fp);                        
          fclose(fp);
          if (externalbody( ctx, 3 ) != -1) {
            syslog(LOG_INFO, "Signatur added %s", priv->temp_mail );
          }
        } else {
          count = 4;
        }
        mysql_free_result(result); 
        break; 
      } else { 
        syslog(LOG_ERR, "mysql_query problem: %s", mysql_error(g_mysql) );
      } 
      count += 1;
      usleep(299999);
      if (mysql_ping(g_mysql) != 0) {
        syslog(LOG_CRIT, "mysql_ping (%d): %s", mysql_errno(g_mysql), mysql_error(g_mysql)); 
      }
    }

    mysql_close( g_mysql );
    if (count > 3)
      return FALSE;   
    else
      return TRUE;
}


/* ----------------------------------------------------------------------------
 * run external anti-virus script
 * ----------------------------------------------------------------------------*/
int 
externalcommand(SMFICTX *ctx, char *workdir, int fd, char *p, char **av) {
    struct mlfiPriv *priv = MLFIPRIV;
    char *ev[]={"PATH=/bin:/usr/bin",NULL};
    int childstatus;
    int pipes[2];
    char *s= NULL;
    FILE *fd_pipe;
    size_t n = 0;

    SAFE_FREE(priv->addbcc);
    SAFE_FREE(priv->newsubject);
    SAFE_FREE(priv->newhead);

    if ((priv->child1pid) != 0)
      syslog(LOG_CRIT,"Script() called and priv->child1pid is %d", priv->child1pid);
    
    pipe(pipes);
    childstatus = 0;
    priv->child1pid = fork();
    switch(priv->child1pid) {
      case -1: /* error */
        syslog(LOG_CRIT, "fork(): %s", strerror(errno));
        priv->child1pid=0;
        return(-1);
        break;
      case 0: /* child */
        if (workdir!=NULL) {
            if (chdir(workdir)!=0) {
              syslog(LOG_CRIT, "Script: chdir(%s): %s", workdir, strerror(errno));
            }
        }
        close(pipes[0]);
        dup2 (pipes[1], STDOUT_FILENO);
        execve(p, av, ev);
        syslog(LOG_CRIT,"Script: failed to execve() %s", p);
        exit(1);
        break;
      default: /* parent */
        close(pipes[1]);
        fd_pipe = fdopen(pipes[0], "r");
        while(getline(&s, &n, fd_pipe) != -1) {
            remove_crln(s);
            syslog(LOG_INFO,"Script: %s ", s);
            if (strncasecmp("BCC:", s, 4) == 0) {
                priv->addbcc = calloc( 1, strlen(s)+1 );
                if (priv->addbcc == NULL) {
                    syslog(LOG_ERR,"Could not allocate memory for addbcc!");
                } else {
                    sscanf(s, "%*s %s", priv->addbcc);
                }
            }
            if (strncasecmp("SUB:", s, 4) == 0) {
                priv->newsubject = calloc( 1, strlen(s)+1 );
                if (priv->newsubject == NULL) {
                    syslog(LOG_ERR,"Could not allocate memory for newsubject!");
                } else {
                    sscanf(s, "%*s %s", priv->newsubject);
                }
            }
            if (strncasecmp("XHD:", s, 4) == 0) {
                priv->newhead = calloc( 1, strlen(s)+1 );
                if (priv->newhead == NULL) {
                    syslog(LOG_ERR,"Could not allocate memory for newsubject!");
                } else {
                    sscanf(s, "%*s %s", priv->newhead);
                }
            }
        }
        SAFE_FREE(s);
        fclose(fd_pipe);

        waitpid(priv->child1pid,&(childstatus), 0);
        priv->child1pid = 0;
        if (WIFEXITED(childstatus)) {
            if (debuglevel > 1)
                syslog(LOG_INFO,"Script: %s return: %d", p, WEXITSTATUS(childstatus));
            return(WEXITSTATUS(childstatus));
        } else {
            syslog(LOG_INFO,"Script: %s return: -1", p);
            return(-1);
        }
        break;
    }
    syslog(LOG_INFO,"externalcommand: %s return: -1", p);
    return(-1);
}


/* ----------------------------------------------------------------------------
 * run altermime - add signatur and remove attachment
 * ----------------------------------------------------------------------------*/
int 
externalbody(SMFICTX *ctx, int nOperation ) {
    struct mlfiPriv *priv = MLFIPRIV;
    int childstatus;
    char buff1[128];
    char buff2[128];
    char buff3[128];
    char buff4[128];
    
    memset(buff1, '\0', 128 );
    memset(buff2, '\0', 128 );
    memset(buff3, '\0', 128 );
    memset(buff4, '\0', 128 );

    snprintf(buff1, 127, "--input=%s", priv->temp_mail);
    if ( nOperation & 1 ) {
      snprintf(buff2, 127, "--disclaimer=%s", priv->temp_signtxt);
      snprintf(buff3, 127, "--disclaimer-html=%s", priv->temp_signhtm);
    }
    if ( nOperation & 2 ) {
      if (optremove)
        snprintf(buff4, 127, "--remove=%s", optremove );
    }
    
    childstatus = 0;
    priv->child2pid = fork();
    switch(priv->child2pid) {
      case -1: /* error */
        syslog(LOG_CRIT, "fork(): %s", strerror(errno));
        priv->child2pid = 0;
        return(-1);
        break;
      case 0: /* child */
        switch(nOperation) {
          case 1:  
          case 3:  
            if (debuglevel > 2)
                syslog(LOG_INFO, "altermime %s %s %s %s", buff1, buff2, buff3, buff4);
            // "--force-for-bad-html"    
            if( execl(ALTERMIME_FILE , "altermime", buff1, buff2, buff3, buff4, NULL ) < 0)
                syslog(LOG_CRIT,"Failed to run: altermime %s %s %s %s", buff1, buff2, buff3, buff4);
          case 2:  
            if (debuglevel > 2)
                syslog(LOG_INFO, "altermime %s %s", buff1, buff4);
            if( execl(ALTERMIME_FILE , "altermime", buff1, buff4, NULL ) < 0)
                syslog(LOG_CRIT,"Failed to run: altermime %s %s", buff1, buff4);
        }
        return(-1);
        break;
      default: /* parent */
        waitpid(priv->child2pid,&(childstatus), 0);
        priv->child2pid = 0;
        if (WIFEXITED(childstatus)) {
            return(WEXITSTATUS(childstatus));
        } else {
            return(-1);
        }
        break;
    }
    return(-1);
}


/* ----------------------------------------------------------------------------
 * check SPF domain entry
 * ----------------------------------------------------------------------------*/
int
spf_check(struct mlfiPriv *priv) {
    char *helo = priv->helo_host;
    char *fromp = priv->from_addr;
    SPF_server_t *spf_server;
    SPF_request_t *spf_request;
    SPF_response_t *spf_response;
    char from[NS_MAXDNAME + 1];
    int result = 0;
    char *domain_pos;
    size_t len;

    if (fromp[0] == '<')
        fromp++;
    strncpy (from, fromp, NS_MAXDNAME);
    from[NS_MAXDNAME] = '\0';
    len = strlen(from);
    if (fromp[len - 1] == '>')
      from[len - 1] = '\0';
    domain_pos = strchr (from, '@');

    if (domain_pos == NULL) {
      syslog(LOG_INFO,"SPF: No domain part in envfrom field -> no SPF check");  
      return 1;
    }

    if ((spf_server = SPF_server_new (SPF_DNS_CACHE, 0)) == NULL) {
      syslog(LOG_ERR,"SPF_server_new failed");
      goto out1;
    }

    if ((spf_request = SPF_request_new (spf_server)) == NULL) {
      syslog(LOG_ERR,"SPF_request_new failed");
      goto out2;
    }

    /*  Get the IP address */
    if (SPF_request_set_ipv4_str(spf_request, priv->conn_addr) && SPF_request_set_ipv6_str(spf_request, priv->conn_addr)) {
      syslog(LOG_ERR,"SPF_request_set_ip_str failed");
      goto out3;
    }    

    /* HELO string */
    if (SPF_request_set_helo_dom (spf_request, helo) != 0) {
      syslog(LOG_ERR,"SPF_request_set_helo_dom failed");
      goto out3;
    }

    if (SPF_request_set_env_from (spf_request, from) != 0) {
      syslog(LOG_ERR,"SPF_request_set_env_from failed");
      goto out3;
    }
    
    if (debuglevel > 2)
      syslog(LOG_INFO,"SPF: info IP: %s helo: %s from: %s", priv->conn_addr, helo, from );

    /*  Get the SPF result  */
    SPF_request_query_mailfrom (spf_request, &spf_response);
    result = SPF_response_result (spf_response);

    SPF_response_free (spf_response);
out3:
    SPF_request_free (spf_request);
out2:
    SPF_server_free (spf_server);
out1:
    return result;
}


/* ----------------------------------------------------------------------------
 * Milter functions:
 * ----------------------------------------------------------------------------*/
sfsistat
mlfi_connect (SMFICTX *ctx, char *hostname, _SOCK_ADDR *hostaddr) {
    struct mlfiPriv *priv;

    /* allocate memory for private data */
    if ((priv = calloc(1, sizeof *priv)) == NULL) {
        syslog(LOG_ERR, "memory allocation failed %s", strerror(errno));
        return SMFIS_ACCEPT;
    }

    /* set private data pointer */
    if (smfi_setpriv(ctx, (void*)priv) != MI_SUCCESS) {
        syslog(LOG_ERR, "can't set private data pointer");
        SAFE_FREE(priv);
        return SMFIS_TEMPFAIL;
    }

    if (!hostaddr) {
        strcpy(priv->conn_addr, "127.0.0.1");
        priv->status = 0;
        return SMFIS_CONTINUE;
    }

    /* store connection data */
    if ( hostaddr->sa_family == AF_INET ) {
        struct sockaddr_in* sa_in;
        sa_in = (struct sockaddr_in *) hostaddr;

        if (!inet_ntop(AF_INET, &sa_in->sin_addr.s_addr, priv->conn_addr, sizeof(priv->conn_addr))) {
            syslog(LOG_ERR, "can't parse connect address: %s", strerror(errno));
            return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
        }
        if ( whitelist != (iptab) 0 ) {
            unsigned char* uchar_addr = (unsigned char*) &sa_in->sin_addr.s_addr;
            ipaddress ipa;
            char str[100];
            ipa.octets[0] = ipa.octets[1] = ipa.octets[2] = ipa.octets[3] = ipa.octets[4] = ipa.octets[5] = ipa.octets[6] = ipa.octets[7] = ipa.octets[8] = ipa.octets[9] = 0;
            ipa.octets[10] = ipa.octets[11] = 0xff;
            ipa.octets[12] = uchar_addr[0];
            ipa.octets[13] = uchar_addr[1];
            ipa.octets[14] = uchar_addr[2];
            ipa.octets[15] = uchar_addr[3];
            ipa.prefixlen = 128;
            if ( iptab_check( whitelist, &ipa ) ) {
                syslog( LOG_INFO, "whitelist \"%s\" [%s]", hostname, iptab_format_address( &ipa, str, sizeof(str) ) );
                priv->whitelisted = 1;
            }
        }
    } else if ( hostaddr->sa_family == AF_INET6 ) {
        struct sockaddr_in6* sa_in6;
        sa_in6 = (struct sockaddr_in6*) hostaddr;

        if (!inet_ntop(AF_INET6, &sa_in6->sin6_addr.s6_addr, priv->conn_addr, sizeof(priv->conn_addr))) {
            syslog(LOG_ERR, "can't parse connect address: %s", strerror(errno));
            return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
        }
        if ( whitelist != (iptab) 0 ) {
            unsigned char* uchar_addr = (unsigned char*) &sa_in6->sin6_addr.s6_addr;
            ipaddress ipa;
            char str[100];
            ipa.octets[0] = uchar_addr[0];
            ipa.octets[1] = uchar_addr[1];
            ipa.octets[2] = uchar_addr[2];
            ipa.octets[3] = uchar_addr[3];
            ipa.octets[4] = uchar_addr[4];
            ipa.octets[5] = uchar_addr[5];
            ipa.octets[6] = uchar_addr[6];
            ipa.octets[7] = uchar_addr[7];
            ipa.octets[8] = uchar_addr[8];
            ipa.octets[9] = uchar_addr[9];
            ipa.octets[10] = uchar_addr[10];
            ipa.octets[11] = uchar_addr[11];
            ipa.octets[12] = uchar_addr[12];
            ipa.octets[13] = uchar_addr[13];
            ipa.octets[14] = uchar_addr[14];
            ipa.octets[15] = uchar_addr[15];
            ipa.prefixlen = 128;
            if ( iptab_check( whitelist, &ipa ) ) {
                syslog( LOG_INFO, "whitelist \"%s\" [%s]", hostname, iptab_format_address( &ipa, str, sizeof(str) ) );
                priv->whitelisted = 1;
            }
        }
    }
    priv->status = 0;
    return SMFIS_CONTINUE;
}


sfsistat
mlfi_helo (SMFICTX *ctx, char *helohost) {
    struct mlfiPriv *priv = MLFIPRIV;
    /* check for private data presented */
    if (!priv) return SMFIS_ACCEPT;

    /* check for required data presented */
    if (!helohost || *helohost == '\0') {
      smfi_setreply(ctx, "501", "5.5.2", "HELO requires domain address");
      return SMFIS_REJECT;
    }

    /* store helo hostname */
    strncpy(priv->helo_host, helohost, MAXLINE - 15);
    return SMFIS_CONTINUE;
}


sfsistat
mlfi_envfrom (SMFICTX *ctx, char **envfrom) {
    struct mlfiPriv *priv = MLFIPRIV;
    char *from_addr;

    /* check for private data presented */
    if (!priv) return SMFIS_ACCEPT;

    /* get macro data */
    if (!(from_addr = smfi_getsymval(ctx, "{mail_addr}"))) {
      syslog(LOG_ERR, "mail_macro: {mail_addr} must be available");
      return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
    }

    /* workaround bogus address in MAIL FROM: <> */
    if (from_addr == '\0') {
      snprintf(priv->from_addr, MAXLINE - 1, "postmaster@%s", priv->helo_host);
      snprintf(priv->report, MAXLINE - 1, "couldn't verify sender");
      priv->status |= STAT_WARN;
      return SMFIS_CONTINUE;
    }

    strtolower(from_addr);
    strncpy(priv->from_addr, from_addr, MAXLINE - 1);

    /* Is the user authenticated? */
    if (smfi_getsymval(ctx, "{auth_type}")) {
      priv->status |= STAT_RELAY;
      return SMFIS_CONTINUE;
    }

    /* check is the sender address SPF-compliant? */
    if ((spfcheck) && ( ! priv->whitelisted)) {
      switch ( spf_check(priv) ) {
        case SPF_RESULT_PASS:
          if (debuglevel > 0)      
            syslog(LOG_INFO, "SPF: PASS %s with %s", priv->from_addr, priv->helo_host );
          break;
        case SPF_RESULT_SOFTFAIL:
          if (debuglevel > 0)
            syslog(LOG_INFO, "SPF: SOFTFAIL %s with %s", priv->from_addr, priv->helo_host );      
          break;
        case SPF_RESULT_NEUTRAL:
        case SPF_RESULT_NONE:
          if (debuglevel > 0)
            syslog(LOG_INFO, "SPF: NONE %s with %s", priv->from_addr, priv->helo_host );      
          break;    
        case SPF_RESULT_FAIL:
          snprintf(priv->report, MAXLINE - 1, "The domain from %s has declared using SPF that it does not send mail through %s. That is why the message was rejected.", priv->from_addr, priv->helo_host );
          syslog(LOG_ERR, "SPF: REJECT %s with %s", priv->from_addr, priv->helo_host );             
          smfi_setreply(ctx, "554", "5.6.1", priv->report);      
          return mlfi_cleanup(ctx, SMFIS_REJECT);        
      }
    }
    return SMFIS_CONTINUE;
}


sfsistat
mlfi_envrcpt (SMFICTX *ctx, char **envrcpt) {
    struct mlfiPriv *priv = MLFIPRIV;
    int fd;

    /* check for private data presented */
    if (!priv) return SMFIS_ACCEPT;

     /* create temp mail file, if not exists */
    if (*priv->temp_mail == '\0') {
        strcpy(priv->temp_mail, TMP_FILE);
        fd = mkstemp(priv->temp_mail);
        if (fd == -1) {
            syslog(LOG_ERR, "mkstemp: can't create: %s", strerror(errno));
            return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
        }
        fchmod(fd, 0644);
        (void) close( fd );
    }   

    return SMFIS_CONTINUE;
}


sfsistat
mlfi_header (SMFICTX *ctx, char *headerf, char *headerv) {
    struct mlfiPriv *priv = MLFIPRIV;
    const char *splitter = ": ";
    int fd, errval;
    ssize_t nwritten;

    /* check for private data presented */
    if (!priv) return SMFIS_ACCEPT;

    /* skip null headers */
    if (*headerf == '\0') return SMFIS_CONTINUE;

    /* get subject if found */
    if(strcasecmp(headerf, "Subject") == 0) {
      priv->subject = strdup( headerv );
      if (debuglevel > 1)
        syslog(LOG_INFO, "Header Subject: %s", priv->subject );
    }
    /* set default to UTF-8 */
    priv->utf8 = 1;
    /* get Content-Type header */
    if (strcasecmp(headerf, "Content-Type") == 0) {
      if ((strstr(headerv, "ISO-" )) || (strstr(headerv, "iso-" )))  {
        priv->utf8 = 0;
      }
      if (debuglevel > 2) {
        remove_crln( headerv );
        syslog(LOG_INFO, "Header Content-Type: %s ", headerv );
      }
    }
 
    if (*priv->temp_mail) {
      if ((fd = open(priv->temp_mail, O_WRONLY|O_APPEND|O_CREAT|O_SYNC)) < 0) {
        syslog(LOG_ERR, "Can't open message tempfile: %s", strerror(errno));
        return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
      }    
      if( file_write( fd, headerf, strlen(headerf), 0, &nwritten, &errval ) == -1 ||
        file_write( fd, splitter, strlen(splitter), 0, &nwritten, &errval ) == -1 ||
        file_write( fd, headerv, strlen(headerv), 0, &nwritten, &errval ) == -1 ||
        file_write( fd, STR_CRLF, strlen(STR_CRLF), 0, &nwritten, &errval ) == -1 ) {
          syslog( LOG_ERR, "Header data write error: %s", strerror(errval) );
          (void) close( fd );
          return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
      }
      (void) close( fd );
    }
    return SMFIS_CONTINUE;
}


sfsistat
mlfi_eoh(SMFICTX *ctx)
{
    struct mlfiPriv *priv = MLFIPRIV; 
    int fd, errval;
    ssize_t nwritten;

    /* output the blank line between the header and the body */
    if (*priv->temp_mail) {
      if ((fd = open(priv->temp_mail, O_WRONLY|O_APPEND|O_CREAT|O_SYNC)) < 0) {
        syslog(LOG_ERR, "Can't open message tempfile: %s", strerror(errno));
        return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
      }    
      if (file_write( fd, STR_CRLF, strlen(STR_CRLF), 0, &nwritten, &errval ) == -1 )
        syslog(LOG_ERR, "%s", strerror(errno));
      priv->body_offset = lseek( fd, 0, SEEK_CUR );
      (void) close( fd );
    }
    return SMFIS_CONTINUE;
}


sfsistat
mlfi_body (SMFICTX *ctx, u_char *bodyp, size_t len) {
    struct mlfiPriv *priv = MLFIPRIV;
    int            fd, errval;
    ssize_t        nwritten;
    
    /* check for private data presented */
    if (!priv) return SMFIS_ACCEPT;
    /* check for required data presented */
    if (!bodyp) {
        syslog(LOG_ERR, "Missing message body");
        return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
    }

    /* open mail temp file (with all header ) */
    if (*priv->temp_mail) {
        if ((fd = open(priv->temp_mail, O_WRONLY|O_APPEND|O_CREAT|O_SYNC)) < 0) {
            syslog(LOG_ERR, "Can't open message tempfile: %s", strerror(errno));
                return mlfi_cleanup(ctx, SMFIS_TEMPFAIL);
        }
        if (file_write( fd, bodyp, len, 0, &nwritten, &errval ) == -1 )
            syslog(LOG_ERR, "can't write mail body message tempfile: %s", strerror(errno));
        (void) close( fd );
    }
    return SMFIS_CONTINUE;
}


sfsistat
mlfi_eom (SMFICTX *ctx) {
    struct mlfiPriv *priv = MLFIPRIV;
    int ret = 0;
    bool lret = FALSE;
    u_int32_t nfilesize;

    /* check for private data presented */
    if (!priv) return SMFIS_ACCEPT;

    /* get current file size before run virus scanner and body modifiers */
    nfilesize = get_file_size(priv->temp_mail);

    if (clamcheck) {
        ret = clamd_check(priv->temp_mail, priv->report, clamsock);
        switch (ret) {
          case CLAMD_OK:
            smfi_addheader(ctx, CLAMD_HEADER, "ClamAV using " MILTER);
            break;
          case CLAMD_PHISHING:
            smfi_addheader(ctx, CLAMD_HEADER, "ClamAV found phishing message" );
            setnewsubject(ctx, priv->report);
            smfi_addrcpt(ctx, avmail );
            return mlfi_cleanup(ctx, SMFIS_CONTINUE);
            break;
          case CLAMD_FOUND:
            file_save_local( quarantinedir, priv->temp_mail, priv->report);
            smfi_setreply(ctx, "554", "5.6.1", priv->report);
            return mlfi_cleanup(ctx, SMFIS_REJECT);
            break;
        }
    }
    if (fprotcheck) {
        ret = fprot_check(priv->temp_mail, priv->report);
        switch (ret) {
          case FPROT_OK:
            smfi_addheader(ctx, CLAMD_HEADER, "F-Prot using " MILTER);
            break;
        case FPROT_FOUND:
            file_save_local( quarantinedir, priv->temp_mail, priv->report);
            smfi_setreply(ctx, "554", "5.6.1", priv->report);
            return mlfi_cleanup(ctx, SMFIS_REJECT);
            break;
        }
    }
    if (scriptfile) {
        char *p = scriptfile;
        char *av[] = {NULL,NULL,NULL,NULL};
        av[0] = scriptfile;
        av[1] = priv->temp_mail;
        ret = externalcommand(ctx, NULL, EOF, p, av);
        switch (ret) {
            case 0:
              smfi_addheader(ctx, CLAMD_HEADER, "Scriptfile   " MILTER);
              syslog(LOG_INFO, "Script: clean  %s", priv->temp_mail );
              break;
            case  9:
              smfi_addheader(ctx, CLAMD_HEADER, "Scriptfile   " MILTER);
              if (priv->addbcc != NULL) {
                  smfi_addrcpt(ctx, priv->addbcc );
                  if (debuglevel > 1)
                      syslog(LOG_INFO, "Add recipient: %s", priv->addbcc );
              }
              if (priv->newsubject != NULL) {
                  setnewsubject(ctx, priv->newsubject);
                  if (debuglevel > 1)
                      syslog(LOG_INFO, "New subject: %s", priv->newsubject );
              }
              if (priv->newhead != NULL) {
                  smfi_addheader(ctx, "X-Header", priv->newhead);
                  if (debuglevel > 1)
                      syslog(LOG_INFO, "New X-Header: %s", priv->newhead );
              }
              break;
            case 10: // virus found!
              if (priv->newsubject != NULL) {
                  snprintf(priv->report, MAXLINE - 1, "content reject %s", priv->newsubject);
                  syslog(LOG_ERR, "Script: content reject %s", priv->newsubject );
              } else {
                  snprintf(priv->report, MAXLINE - 1, "content reject");
                  syslog(LOG_ERR, "Script: content reject" );
              }
              file_save_local( quarantinedir, priv->temp_mail, priv->report);
              smfi_setreply(ctx, "554", "5.6.1", priv->report);
              return mlfi_cleanup(ctx, SMFIS_REJECT);
              break;
            case 11: // user unknown
              if (priv->newsubject != NULL) {
                  snprintf(priv->report, MAXLINE - 1, "User unknown %s", priv->newsubject);
                  syslog(LOG_ERR, "Script: user unknown %s", priv->newsubject );
              } else {
                  snprintf(priv->report, MAXLINE - 1, "User unknown");
                  syslog(LOG_ERR, "Script: user unknown");
              }
              file_save_local( quarantinedir, priv->temp_mail, priv->report);
              smfi_setreply(ctx, "550", "5.1.1", priv->report);
              return mlfi_cleanup(ctx, SMFIS_REJECT);
              break;
        }
    }
    
    /***************** body replace! ******************************************/
    if (priv->status & STAT_RELAY){
        if (signatureadd) 
            lret = add_external_signature(ctx);
        if ((optremove) && (lret == FALSE)) {        
                ret = externalbody(ctx, 2);
                switch (ret) {
                    case 1:
                      syslog(LOG_ERR,  "ERR body rem.: %s attachment: %s ", priv->temp_mail, optremove );
                      break;
                    case 9:
                      syslog(LOG_INFO, "Remove from:   %s attachment: %s ", priv->temp_mail, optremove );
                      break;
                }
        }
    } 
    /* if file size changed then replace mail message body */   
    if (nfilesize != get_file_size(priv->temp_mail))
        change_body(ctx);
    return mlfi_cleanup(ctx, SMFIS_CONTINUE);
}


sfsistat
mlfi_cleanup (SMFICTX *ctx, sfsistat rc) {
    struct mlfiPriv *priv = MLFIPRIV;

    /* check for private data presented */
    if (!priv) return SMFIS_CONTINUE;

    if ((priv->child1pid)>0) {
        syslog(LOG_ERR, "cleanup() killing %d av-script", priv->child1pid);
        kill(priv->child1pid, SIGABRT);
        waitpid(priv->child1pid, NULL, 0);
        priv->child1pid = 0;
    }
    if ((priv->child2pid)>0) {
        syslog(LOG_ERR, "cleanup() killing %d altermime", priv->child2pid);
        kill(priv->child2pid, SIGABRT);
        waitpid(priv->child2pid, NULL, 0);
        priv->child2pid = 0;
    }
    if (debuglevel < 4) {
        /* unlink temporary files */
        if (*priv->temp_mail) {
              if (debuglevel > 2) {
                syslog(LOG_INFO, "Cleanup remove %s", priv->temp_mail);
              }
          (void) unlink(priv->temp_mail);
        }
        if (*priv->temp_signtxt)
            (void) unlink(priv->temp_signtxt);
        if (*priv->temp_signhtm)
            (void) unlink(priv->temp_signhtm);
    }
    /* cleanup per-message data */
    *priv->from_addr    = '\0';
    *priv->temp_mail    = '\0';
    *priv->temp_signtxt = '\0';
    *priv->temp_signhtm = '\0';
    *priv->report       = '\0';
    *priv->helo_host    = '\0';
    SAFE_FREE(priv->subject);
    SAFE_FREE(priv->addbcc);
    SAFE_FREE(priv->newsubject);
    SAFE_FREE(priv->newhead);
    priv->body_offset   = 0;
    /* release private memory */
    free(priv);
    smfi_setpriv(ctx, NULL);
    return rc;
}


sfsistat
mlfi_abort (SMFICTX *ctx) {
    return mlfi_cleanup(ctx, SMFIS_CONTINUE);
}


sfsistat
mlfi_close (SMFICTX *ctx) {
    return mlfi_cleanup(ctx, SMFIS_CONTINUE);
}


struct smfiDesc smfilter = {
    MILTER,          /* filter name */
    SMFI_VERSION,    /* version code -- do not change */
    SMFIF_ADDHDRS|SMFIF_CHGBODY|SMFIF_ADDRCPT|SMFIF_DELRCPT|SMFIF_CHGHDRS|SMFIF_QUARANTINE,    /* flags */
    /* flags */
    mlfi_connect,    /* connection info filter */
    mlfi_helo,       /* SMTP HELO command filter */
    mlfi_envfrom,    /* envelope sender filter */
    mlfi_envrcpt,    /* envelope recipient filter */
    mlfi_header,     /* header filter */
    mlfi_eoh,        /* end of header */
    mlfi_body,       /* body block filter */
    mlfi_eom,        /* end of message */
    mlfi_abort,      /* message aborted */
    mlfi_close,      /* connection cleanup */
    NULL,            /* unknown SMTP commands */
    NULL,            /* DATA command */
    NULL             /* Once, at the start of each SMTP connection */
};


/* MySQL handling 
#######################################################################
# Database functions
#######################################################################
*/

bool
mydb_test( void ) {
    char testbuffer[1024];
    MYSQL *g_mysql;

    if(!dbname){
        syslog(LOG_ERR, "DBName not defined");
        return FALSE;
    }
    else if(!dbhost){
        syslog(LOG_ERR, "DBServer not defined");
        return FALSE;
    }
    else if(!dbuser){
        syslog(LOG_ERR, "DBUser not defined");
        return FALSE;
    }
    else if(!dbtable){
        syslog(LOG_ERR, "DBTable not defined");
        return FALSE;
    }
    g_mysql= mysql_init( NULL );
    
    if ( ! mysql_real_connect(g_mysql, dbhost, dbuser, dbpass, dbname, dbport, 0, 0)) {
        syslog(LOG_ERR, "db_test: mysql_real_connect failed");
        syslog(LOG_ERR, "%s", mysql_error(g_mysql));
        mysql_close(g_mysql);
        return FALSE;
    }

    snprintf(testbuffer, 1023, "SELECT signature FROM %s LIMIT 1", dbtable);
    /* run the query check */
    if (mysql_query(g_mysql, testbuffer ) != 0)
        syslog(LOG_ERR, "db_query problem: %s", mysql_error(g_mysql));

    mysql_close(g_mysql);
    return TRUE;
}


/* ----------------------------------------------------------------------------
 * load / reload configuration
 * ----------------------------------------------------------------------------*/
int
read_config (void) {
    FILE *fh = fopen(CONFIG_FILE, "r");
    char buffer[MAXLINE], value[MAXLINE];

    if (!fh)
        return -1;

    while (fgets(buffer, MAXLINE, fh))
    if (*buffer == '#') {
        continue;
    } else if (strncasecmp("user", buffer, 4) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!smfiuser) smfiuser = strdup(value);
    } else if (strncasecmp("socket", buffer, 6) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!smfisock) smfisock = strdup(value);
    } else if (strncasecmp("clamcheck", buffer, 9) == 0) {
        sscanf(buffer, "%*s %s", value);
        clamcheck = strcasecmp(value, "yes")? 0 : 1;
    } else if (strncasecmp("clamsocket", buffer, 10) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!clamsock) clamsock = strdup(value);
    } else if (strncasecmp("fprotcheck", buffer, 9) == 0) {
        sscanf(buffer, "%*s %s", value);
        fprotcheck = strcasecmp(value, "yes")? 0 : 1;
    } else if (strncasecmp("quarantinedir", buffer, 13) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!quarantinedir) quarantinedir = strdup(value);
    } else if (strncasecmp("avmail", buffer, 6) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!avmail) avmail = strdup(value);
    } else if (strncasecmp("signatureadd", buffer, 12) == 0) {
        sscanf(buffer, "%*s %s", value);
        signatureadd = strcasecmp(value, "yes")? 0 : 1;
    } else if (strncasecmp("spfcheck", buffer, 8) == 0) {
        sscanf(buffer, "%*s %s", value);
        spfcheck = strcasecmp(value, "yes")? 0 : 1;
    } else if (strncasecmp("dbhost", buffer, 6) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!dbhost) dbhost = strdup(value);
    } else if (strncasecmp("dbport", buffer, 6) == 0) {
        sscanf(buffer, "%*s %d", &dbport);
    } else if (strncasecmp("dbname", buffer, 6) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!dbname) dbname = strdup(value);
    } else if (strncasecmp("dbtable", buffer, 7) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!dbtable) dbtable = strdup(value);
    } else if (strncasecmp("dbuser", buffer, 6) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!dbuser) dbuser = strdup(value);
    } else if (strncasecmp("dbpass", buffer, 6) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!dbpass) dbpass = strdup(value);
    } else if (strncasecmp("optremove", buffer, 9) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!optremove) optremove = strdup(value);
    } else if (strncasecmp("debuglevel", buffer, 10) == 0) {
        sscanf(buffer, "%*s %d", &debuglevel);
    } else if (strncasecmp("scriptfile", buffer, 10) == 0) {
        sscanf(buffer, "%*s %s", value);
        if (!scriptfile) scriptfile = strdup(value);
    }

    if (!dbhost)   dbhost   = strdup("localhost");
    if (!smfiuser) smfiuser = strdup(DEFAULT_USER);
    if (!smfisock) smfisock = strdup(SOCKET_FILE);
//    if (!smfisock) smfisock = strdup(SOCKET_PORT);
    if (!clamsock) clamsock = strdup(CLAMD_SOCKET_FILE);
    if (!quarantinedir) quarantinedir = strdup("/var/tmp");
    if (!avmail)   avmail   = strdup("root@localhost");
   

    fclose(fh);
    return 0;
}


/* ----------------------------------------------------------------------------
 * insert data to whitelist
 * ----------------------------------------------------------------------------*/
static void
init_whitelist( const char* whitelist_filename ) {
    FILE* fp;
    char line[500];
    ipaddress ipa;

    if ( whitelist_filename != (char*) 0 ) {
        whitelist = iptab_new();
        if ( whitelist == (iptab) 0 ) {
            syslog( LOG_ERR, "whitelist create failed" );
            exit( EX_OSERR );
        }
        fp = fopen( whitelist_filename, "r" );
        if ( fp == (FILE*) 0 ) {
            syslog( LOG_ERR, "fopen '%s' - %m", whitelist_filename );
            exit( EX_OSERR );
        }
        while ( fgets( line, sizeof(line), fp ) != (char*) 0 ) {
            utiltrim( line );
            if (( line[0] == '\0' ) || ( line[0] == '#' ))
                continue;
            if ( iptab_parse_address( line, &ipa ) )
               (void) iptab_add( whitelist, &ipa );
            else
               syslog( LOG_WARNING, "unparsable IP address - '%s'", line );
        }
        (void) fclose( fp );
    }
    else
       whitelist = (iptab) 0;
}

/* ----------------------------------------------------------------------------
 * clear whitelist
 * ----------------------------------------------------------------------------*/
static void
term_whitelist( void ) {
    if ( whitelist != (iptab) 0 )
        iptab_delete( whitelist );
}


static void
usage () {
    printf("Usage:\t" PACKAGE "\t[-fhv]\n\n");
}

static void
version () {
    printf(PROJECT_NAME " version " VERSION "\n" COPYRIGHT "\n\n");
}

static void
help () {
    version();
    usage();
    printf("\t-v\t\t\tShow program version.\n");
    printf("\t-h\t\t\tShow this help.\n\n");
    printf("Program recognises the following config file options:\n\n");
    printf("\t[user username]\t\tSpecifies the user the milter should\n");
    printf("\t\t\t\trun as after it initializes.\n");
    printf("\t\t\t\tmail which has not passed the filter checks.\n");
    printf("\t[socket path]\t\tPath to create socket.\n");
    printf("\t[dbname name]\t\tName of MySQL database.\n");
    printf("\t[dbtable name]\t\tName of MySQL table.\n");
    printf("\t[dbuser name]\t\tName of MySQL connect user.\n");
    printf("\t[dbpass name]\t\tPassword to connect MySQL table.\n");
    printf("\t[clamcheck (yes|no)]\tEnable or disable mail checks\n");
    printf("\t\t\t\tusing ClamAV antivirus.\n");
    printf("\t[clamsocket path]\tPath to ClamAV socket.\n");
    printf("\t[quarantinedir path]\tPath for infected files.\n");
    printf("\t[avmail mailuser]\t\tE-Mail address for infect messages.\n");

    printf("Program settings:\n\n");
    printf("\tuser\t\t\t[%s]\n", smfiuser);
    printf("\tsocket\t\t\t[%s]\n", smfisock);
    printf("\tdbname\t\t\t[%s]\n", dbname);
    printf("\tdbtable\t\t\t[%s]\n", dbtable);
    printf("\tclamcheck\t\t[%s]\n", (clamcheck) ? "yes" : "no");
    printf("\tclamsocket\t\t[%s]\n", clamsock);
    printf("\tfprotcheck\t\t[%s]\n", (fprotcheck) ? "yes" : "no");
    printf("\tquarantinedir\t\t[%s]\n", quarantinedir);
    printf("\tdebuglevel\t\t[%d]\n", debuglevel);

    printf("%s\n\n", DEVELOPERS);
}


/* ----------------------------------------------------------------------------
 * create a pid file
 * ----------------------------------------------------------------------------*/
int
create_pid (const char *pidfile, pid_t pid) {
    char buffer[16];
    pid_t spid;
    int fd;

    if (access(pidfile, R_OK) == 0) {
        if ((fd = open(pidfile, O_RDONLY)) < 0)
            return -1;
        if (read(fd, buffer, sizeof(buffer)) < 0) {
            close(fd);
            return -1;
        }
        close(fd);
        sscanf(buffer, "%d", &spid);
        if (spid == pid)
            return 0;
        if ((kill(spid, 0) < 0) && errno == ESRCH)
            unlink(pidfile);
        else
            return 1;
    }
    if (!pid)
        return 0;
    if ((fd = open(pidfile, O_RDWR|O_TRUNC|O_CREAT, 0644)) < 0)
        return -1;
    /* put my pid in it */
    sprintf(buffer, "%d", pid);
    if (write(fd, buffer, strlen(buffer)) < 0)
        syslog(LOG_ERR, "%s", strerror(errno));
    close(fd);
    return 0;
}



/* ----------------------------------------------------------------------------
 * main
 * ----------------------------------------------------------------------------*/
int
main (int argc, char **argv) {
    int c;
    extern int optind;
    struct passwd *pw;
    pid_t pid;

    /* get configuration file options */
    if (read_config() < 0) {
        fprintf(stderr, "Can't read config file: %s\n", CONFIG_FILE);
        exit(EXIT_FAILURE);
    }

    /* process command line options */
    while ((c = getopt(argc, argv, "fhv:")) != -1) {
        switch (c) {
        case 'h':
        help();
        exit(EX_OK);
        case 'v':
        version();
        exit(EX_OK);
        default:
        usage();
        exit(EX_USAGE);
        }
    }

    if ((pw = getpwnam(smfiuser)) == NULL) {
        fprintf(stderr, "%s: user '%s' not found\n", PACKAGE, smfiuser);
        exit(EXIT_FAILURE);
    }

    init_whitelist( HOSTS_FILE );

    // avoid running as root user and/or group
    if (getuid() == 0 && pw->pw_uid != 0 && pw->pw_gid != 0) {
        (void) setgid(pw->pw_gid);
        (void) setuid(pw->pw_uid);
    }

    // initialize MySQL library specifically now, as it's not thread-safe and
    // might otherwise cause mysql_init() to fail ... saving mutex on the
    // mysql_init this way ...
    if (mysql_library_init(0,NULL,NULL)) {
	    fprintf(stderr,"MySQL library init failed\n");
	    goto CLEANUP;
    }

    // check pid file
    switch (create_pid(PID_FILE, 0)) {
    case -1:
        fprintf(stderr, "%s: can't create pid file: %s\n", PACKAGE, PID_FILE);
        goto CLEANUP;
        break;
    case 1:
        fprintf(stderr, "%s: filter is already running..\n", PACKAGE);
        exit(EX_OK);
        break;
    }

    // run in background
    if ((pid = daemon(0, 0)) < 0) {
        fprintf(stderr, "%s: could not run filter in background, %s", PACKAGE, strerror(errno));
        goto CLEANUP;
    }
    if (pid != 0)
        exit(EX_OK);

    // open syslog
    openlog(PACKAGE, LOG_PID, LOG_MAIL);

    if( (signatureadd) && (mydb_test())){
        syslog(LOG_INFO, "MySQL server %s connect and query check, with table %s ", dbhost, dbtable);
    }
    // get new pid
    pid = getpid();
    // write pid to file
    if (create_pid(PID_FILE, pid)) {
        syslog(LOG_ERR, "can't create pid file %s", PID_FILE);
        goto CLEANUP;
    }
    syslog(LOG_INFO, "running as user '%s'", pw->pw_name);
  
  
 //   if (smfi_settimeout(SOCKET_TIMEOUT) == MI_FAILURE)
 //       exit(EX_UNAVAILABLE);
    if (smfi_setconn(smfisock) == MI_FAILURE) {
        syslog(LOG_ERR, "could not set milter socket");
        goto CLEANUP;
    }
    if (smfi_register(smfilter) == MI_FAILURE) {
        syslog(LOG_ERR, "could not register milter");
        goto CLEANUP;
    }

    smfi_main();
    goto CLEANUP;


CLEANUP:
    mysql_library_end();
    term_whitelist();
    SAFE_FREE(smfiuser);
    SAFE_FREE(smfisock);
    SAFE_FREE(clamsock);
    SAFE_FREE(dbhost);
    SAFE_FREE(dbname);
    SAFE_FREE(dbtable);
    SAFE_FREE(dbuser);
    SAFE_FREE(dbpass);
    SAFE_FREE(optremove);
    SAFE_FREE(scriptfile);

    closelog();
    exit(EX_OK);
}

/* eof */
